      program af2_configure
C+------------------------------------------------------------------------------
C
C     Name:   
C        af2_configure
C
C     Description:
C        Main routine for allocating fibres to for autofib-2. The main
C        output is a mapping of fibre --> object for as many objects
C        in the field as the user deems possible/desirable.  An interactive
C        editor is also supplied.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        None
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        51, 52, 53
C
C     External routines called:
C        Library subaf2_configure:
C           csub_getenv
C           edit_mode
C           getval_c
C           getval_i
C           getval_l
C           getval_r
C           newfield
C           opnewfile
C           placefibs
C           rdfibinfo
C           rdold
C           read_objects
C           refine
C           rep_pl
C           sumweights
C           telescopeinfo
C           write_config
C           xyout
C
C        Library chr:
C           chr_len
C           chr_puti
C
C     Algorithm:
C       -- The telescope information and the field file (the file with
C          the coordinates of the objects to be allocated) is read in.
C          Then the file with the latest information on the fibre bundle
C          being used is read in.
C       -- The user may create a new configuration from scratch or may
C          use an old one as a starting point.
C       -- In order to help avoid collisions, the user may artifically 
C          increase the size of the fibres.
C       -- If using an old configuration, then it is read in.  If not, then
C          a configuration is done using one of the two placement algorithms
C          which are currently on offer.
C       -- The user can call the iteration algorithm in order to improve 
C          the intial placement.  Once this is done, an attempt is made
C          to place any spare fibres on objects with zero weight.
C       -- The placement can be plotted and edited interactively.
C       -- An output configuration file is written. The user may optionally
C          create a file with the x,y coordinates of all the objects placed.
C          In addition, a new version of the input field file can be generated
C          which has all of the allocated objects with a weight of zero.  This
C          can then be used for a subsequent run of this program in order to
C          preferentially allocated objects which weren't allocated before.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global Parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FSTA.INC'
C
C     Local parameters
C
      integer config,logfile,fldfile
      parameter (config=51,logfile=52,fldfile=53)
C
C     Local variables
C
      integer status,len1,i,j,iostat,nold,len2,nobj,nfids,nsky
      integer otype(maxobj),sotype(maxobj),npivots,nguides,piv_r,park_r
      integer fibdims(3),fibstatus(nfibres),piv(2,nfibres)
      integer fib(2,nfibres),park(2,nfibres),allocated(nfibres)
      integer object_ref(maxobj),sky_ref(maxobj),n_obj_left,n_sky_left
      integer nall,len3,nskip,nunused,ialg
      character dir*80,string*80,basename*80,flddir*80,cfgfile*80
      character ocfgfile*80,plog(maxobj)*80,slog(maxobj)*80
      character fibstring*80,label*80,fibrefile*80,xyfile*80
      character nfldfile*80,temp1*10,temp2*10
      logical exist,ifold,ifplot,ifxy,ifnewf,iterate,verbose
      logical whole,replace_pa,hush
      real scales(3),field_size,object(2,maxobj),skypos(2,maxobj)
      real weight(maxobj),sweight(maxobj),fibbuf,maxnonrad,fibrelen2
      real weightsum,new_pa
C
C     Functions
C
      integer chr_len
      real sumweights
C
C     Common blocks
C
      common /scan/ replace_pa,new_pa,hush
C
C     Initialise the status variable
C
      status = af2_ok
C
C     Get the name of the directory where the "general" files are kept
C     These include the fibre information file and the telescope information
C     file. Make sure there is a / at the end of the string.  Note this will
C     only work for UNIX boxes.
C
      call csub_getenv('WHT_AF2DATA_DIR',dir,15,80,status)
      len1 = chr_len(dir)
      if (dir(len1:len1) .ne. '/') dir(len1+1:len1+1) = '/'
C
C     Get the parameters which map the telescope distortion from telescope
C     information file.
C
      call telescopeinfo(dir,field_size,scales,status)
      if (status .ne. af2_ok) go to 500
C
C     Get the input field file.  This should be a file with the extension
C     .fld, but strip this extension off if it does (it will be added back
C     in later).  Also, keep a copy of the base name which you will use 
C     as a base name for other files.  
C
      call getval_c('Input field file',' ',string,status)
      if (status .ne. af2_ok) go to 500
      len1 = chr_len(string)
      if (string(len1-3:len1) .eq. '.fld') len1 = len1 - 4
      basename = string(1:len1)
C
C     Get the name of the directory for later use.  Note this will only work
C     for UNIX files since it searches only for '/' as the directory
C     delimiter.
C
      j = 0
      do 10 i = 1,len1
         if (string(i:i) .eq. '/') j = i
  10  continue
      if (j .gt. 0) then
         flddir = string(1:j)
      else
         flddir = ' '
      end if
C
C     Add the .fld (always lower case!) extension and try to open the 
C     field file now
C
      string(len1+1:len1+4) = '.fld'
      len1 = len1 + 4
      write(*,fmt='(a,a)') 'Input field file is: ',string(:len1)
      open(unit=fldfile,file=string(:len1),status='old',
     :   access='sequential',iostat=iostat)
      if (iostat .ne. 0) then
         status = af2_err
         write(*,fmt='(a,a)') 'Error opening file: ',string(:len1)
         go to 500
      end if
C
C     Decide whether to operate in verbose mode.  Having verbose switched
C     off should cut out most of the messages you get with the exception
C     of error messages.
C
      call getval_l('Do you want verbose output?','no',verbose,status)
      if (status .ne. af2_ok) go to 500
C
C     Form the configure file name and see if there are any old versions
C     of this file lying around.  If there are, then give the user the
C     option of starting his/her/its configure from there.  Note that
C     'configure' files have the extension .cfg.
C
      len1 = chr_len(basename)
      cfgfile = basename(:len1)//'.cfg.1'
      inquire(file=cfgfile(:len1+6),exist=exist)
      if (exist) then
         call getval_l('Use an old configuration file?','no',ifold,
     :      status)
         if (status .ne. af2_ok) go to 500
      else
         ifold = .false.
      end if
C
C     If the user wants to use an old configure file, the find the
C     latest version number.
C
      if (ifold) then
         nold = maxver + 1
         exist = .false.
         do while (.not. exist)
            nold = nold - 1
            cfgfile = basename(:len1)//'.cfg.'
            len2 = len1 + 5
            call chr_puti(nold,cfgfile,len2)
            inquire(file=cfgfile(:len2),exist=exist)
         end do
C
C        Now prompt the user for the version number to use.  If there is only
C        one version, then don't bother to prompt.
C
         if (nold .gt. 1) then
            len2 = 0
            call chr_puti(nold,temp1,len2)
            len3 = 0
            call chr_puti(nold,temp2,len3)
            call getval_i('Which version do you want to use?',
     :         temp1(:len2),'1',temp2(:len3),nold,status)
            if (status .ne. af2_ok) go to 500
         else
            nold = 1
         end if
C
C        Form old configuration file name
C
         ocfgfile = basename(:len1)//'.cfg.'
         len2 = len1 + 5
         call chr_puti(nold,ocfgfile,len2)
      end if
C
C     Open a new configuration file now.
C
      call opnewfile(basename,'cfg',config,.true.,status)
      if (status .ne. af2_ok) go to 500
C
C     Open a log file now...
C
      call opnewfile(basename,'log',logfile,.true.,status)
      if (status .ne. af2_ok) go to 500
C
C     Read the field file.  This includes reading the header information
C     (UTDATE, HA, etc), reading the object information, converting the
C     RA,DEC to x,y and sorting the objects by types.  We are finished
C     with the log file so close it up.  Finally, tell the user how
C     many objects of each type there were.  Note that for convenience
C     sake the number of fiducials is also included in the counter for
C     the number of objects, since they will be allocated simultaneously.
C
      replace_pa = .false.
      hush = .false.
      call read_objects(fldfile,config,logfile,verbose,scales,
     :   field_size,nobj,nfids,nsky,nskip,object,otype,skypos,
     :   sotype,plog,slog,weight,sweight,fibstring,label,status)
      if (status .ne. af2_ok) go to 500
      close(logfile)
      write(*,fmt='(a)') ' '
      write(*,fmt='(i3,a)') nfids,' fiducial objects'
      write(*,fmt='(i3,a)') nobj-nfids,' program objects'
      write(*,fmt='(i3,a)') nsky,' sky positions'
      write(*,fmt='(i3,a)') nskip,' objects outside of field'
      write(*,fmt='(a)') ' '
C
C     If there are fewer than 3 fiducials send out a warning message.
C
      if (nfids .lt. 3) then
         write(*,fmt='(a)') 
     :      '***Warning -- less than 3 fiducials found in field file'
         write(*,fmt='(a)') ' '
      end if
C
C     Form the name of the fibre information file from the name of the fibre
C     bundle as it was read in the header of the field file. The name of
C     the fibre information file is (always in lower case) the name of the
C     bundle with the string '_fibres.dat' appended.  Don't forget the
C     name of the directory as well
C
      len1 = chr_len(fibstring)
      len2 = chr_len(dir)
      fibrefile = dir(:len2)//fibstring(:len1)//'_fibres.dat'
      len1 = len1 + len2 + 11
C
C     See if the fibre info file exists.  If it doesn't, then signal
C     an error
C
      inquire(file=fibrefile(:len1),exist=exist)
      if (.not. exist) then
         write(*,fmt='(a,a,a)') 'The file ',fibrefile(:len1),
     :      ' does not exist'
         write(*,fmt='(a)')
     :      'Do you have the correct bundle name in field file?'
         status = af2_err
         go to 500
      end if
C
C     Get a buffer factor for the fibre dimensions.  This can be used as 
C     insurance against collisions by augmenting the physical sizes of the 
C     button and the tubing and hence making slightly dubious fibre-object
C     combinations less likely.  Note this factor must be greater than or
C     equal to 1.  
C
      call getval_r('Buffer factor for fibre dimensions','1.50','1.00',
     :   ' ',fibbuf,status)
      if (status .ne. af2_ok) go to 500
C
C     Read the fibre information.  This includes the number of pivots,
C     the number of guide fibres, the maximum non-radial angle the fibres
C     can subtend, the square of the fibre length, the dimensions of the
C     fibre tubing and button, the pivot and park radii, the status of
C     each fibre (whether it's disabled, etc), the pivot locations,
C     the current button locations and the park locations.
C
      call rdfibinfo(fibrefile,fibbuf,npivots,nguides,nunused,maxnonrad,
     :   fibrelen2,fibdims,piv_r,park_r,fibstatus,piv,fib,park,status)
      if (status .ne. af2_ok) go to 500
C
C     Initialise the fibre allocation record.  These will eventually 
C     say to which object a specific fibre has been allocated.  If the
C     value is zero, then the fibre is unallocated and available.  If the value
C     is negative then there is something wrong.  The fibre status 
C     (fibstatus) and the allocation record (allocated) can assume the 
C     following values initially.
C
C        fibstatus          allocated       comment
C        ------------------------------------------------------------------
C        fsta_miss            -30           Missing fibre
C        fsta_ook               0           Normal object fibre
C        fsta_fok               0           Normal guide fibre
C        fsta_odis            -10           Disabled object fibre
C        fsta_fdis            -10           Disabled guide fibre
C        fsta_odead           -20           Dead object fibre
C        fsta_fdead           -20           Dead guide fibre
C
C     Later when the fibre allocation has been done, the fibres which
C     started with values of 0 can have the reference number of an object 
C     or sky patch.  
C
      do 20 i = 1,npivots
         if ((fibstatus(i) .eq. fsta_ook) .or. 
     :      (fibstatus(i) .eq. fsta_fok)) then
            allocated(i) = 0
         else if ((fibstatus(i) .eq. fsta_odis) .or. 
     :      (fibstatus(i) .eq. fsta_fdis)) then
            allocated(i) = -10
         else if ((fibstatus(i) .eq. fsta_odead) .or. 
     :      (fibstatus(i) .eq. fsta_fdead)) then
            allocated(i) = -20
         else if (fibstatus(i) .eq. fsta_miss) then
            allocated(i) = -30
         end if
  20  continue
C
C     The array object_ref contains the number of each object.  Initially
C     this is just the number 1,2,3...nobj.  But this array will be 
C     swapped around a lot, so referencing object information won't be 
C     quite so trivial.  Do the same for the sky patches.  
C
      do 30 i = 1,nobj
         object_ref(i) = i
  30  continue
      do 40 i = 1,nsky
         sky_ref(i) = i
  40  continue
C
C     The variables n_obj_left and n_sky_left denote the number of objects
C     and skies which haven't yet been covered by a fibre.  We initialise them
C     to the number of objects and skies read in the field file.
C
      n_obj_left = nobj
      n_sky_left = nsky
C
C     Write out a helpful line of information
C
      if (verbose) 
     :   write(*,fmt='(a,i3,a,i3,a)') 'Trying to configure ',nobj,
     :   ' objects and ',npivots,' pivots'
C
C     ***Beginning of 'ifold' block
C
C     If you are getting an initial configuration from an old configuration
C     file, then read that here.
C
      if (ifold) then
         call rdold(ocfgfile,npivots,fibstatus,verbose,fib,piv,
     :      allocated,object_ref,object,skypos,n_obj_left,n_sky_left,
     :      sky_ref,plog,slog,fibrelen2,maxnonrad,fibdims,status)
         if (status .ne. af2_ok) go to 500
C
C     If you are not using an old configuration then you need to decide which
C     algorithm you want to use.  Currently there is the FAST algorithm
C     (fibres placed by geometry alone) and the SLOW algorithm (fibres
C     placed to try and maximise the sum of weights). Decide which one
C     you want to use.
C
      else
         write(*,fmt='(a)') ' '
         write(*,fmt='(a)') 'Which fibre placement algorithm?'
         write(*,fmt='(a)') '(1) The FAST algorithm'
         write(*,fmt='(a)') '(2) The SLOW algorithm'
         call getval_i('Choose one','2','1','2',ialg,status)
         if (status .ne. af2_ok) go to 500
C
C        Place the fibres on the objects.
C
         call placefibs(npivots,piv,fib,fibstatus,allocated,0,verbose,
     :      n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :      fibdims,fibrelen2,ialg,.true.,nall,status)
         if (status .ne. af2_ok) go to 500
C
C        Report on the number of fibres placed and the total weight of
C        those objects
C
         weightsum = sumweights(npivots,allocated,weight)
         write(*,fmt='(a)') ' '
         write(*,fmt='(a,i3)') 'Total number of objects allocated: ',
     :      nall
         write(*,fmt='(a,1pg10.3)') 
     :      'Total sum of weights of allocated objects: ',weightsum
C
C        Now we need to do that same thing for the dedicated sky patches.
C
         call placefibs(npivots,piv,fib,fibstatus,allocated,alloc_sky,
     :      verbose,n_sky_left,skypos,sweight,slog,sotype,sky_ref,
     :      maxnonrad,fibdims,fibrelen2,ialg,.true.,nall,status)
         if (status .ne. af2_ok) go to 500
C
C        Report on the number of fibres placed on skies 
C
         write(*,fmt='(a,i3)') 
     :      'Total number of sky patches allocated: ',nall
         write(*,fmt='(a)') ' '
C
C      *** End of 'ifold' block
C
      end if
C
C     Give the user a listing of the number of each type of object allocated.
C
      if (verbose) call rep_pl(npivots,allocated,fibstatus,status)
      if (status .ne. af2_ok) go to 500
C
C     In this next section the user can call an iteration procedure which will
C     attempt to improve the placement as it currently stands.  This is done
C     by trying to either park or move a fibre to a different object in the
C     hope that it will either free up more objects or it will increase the
C     total sum of weights of the objects covered.  Note also that this 
C     only moves around object+fiducial allocations.  Sky patches are 
C     left alone.
C
      call getval_l('Iterate the placement?','yes',iterate,status)
      if (status .ne. af2_ok) go to 500
C
C     If you have requested an iteration then try and refine
C     the fibre placement.  Give the user a new list of fibre placings
C
      if (iterate) then
         call refine(npivots,piv,fib,n_obj_left,maxnonrad,fibdims,
     :      fibrelen2,object,fibstatus,otype,allocated,object_ref,
     :      park,weight,plog,verbose,status)
         if (verbose) call rep_pl(npivots,allocated,fibstatus,status)
         if (status .ne. af2_ok) go to 500
      end if
C
C     If you have any objects with weights <= 0.0 and there are some spare
C     fibres to be had, then you might as well try and use them.  So use
C     the SLOW algorithm to try and allocate spare fibres on such objects.
C
      write(*,fmt='(a)') ' '
      write(*,fmt='(a)') 
     :   'Trying to place spare fibres on objects with weight <= 0.0'
      call placefibs(npivots,piv,fib,fibstatus,allocated,0,verbose,
     :   n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :   fibdims,fibrelen2,1,.false.,nall,status)
      if (status .ne. af2_ok) go to 500
C
C     Give the user one last listing of the number of placed objects...
C
      call rep_pl(npivots,allocated,fibstatus,status)
      if (status .ne. af2_ok) go to 500
C
C     Now give the user the opportunity to see the allocation and possibly
C     to edit the configuration manually.  If a plot is being done ask if
C     the user wants to see the whole plate (out to the pivot circle).  
C     Usually that should not be the case.
C
      call getval_l('Plot configuration to allow for editing?','yes',
     :   ifplot,status)
      if (ifplot) then
         call getval_l('Show whole plate?','no',whole,status)
         call edit_mode(nobj,npivots,nsky,object,object_ref,otype,
     :      plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,
     :      piv,park,fibstatus,allocated,label,piv_r,park_r,field_size,
     :      maxnonrad,fibrelen2,fibdims,whole,status)
      end if
      if (status .ne. af2_ok) go to 500
C
C     Ok, that's your last chance of placing fibres for this run.  Write
C     out the configuration (.cfg) file and close it up.
C
      call write_config(config,npivots,allocated,fib,plog,slog,
     :   fibstatus,nfids,nobj,otype,scales,status)
      if (status .ne. af2_ok) go to 500
      close(config)
C
C     Write out a file with the x,y coordinates as they have been calculated
C     in this routine.  This isn't necessary for observing, but is quite
C     useful for comparison with the results of af2_setup.
C
      call getval_l('Generate an (x,y) file?','no',ifxy,status)
      if (ifxy) then
         len1 = chr_len(basename)
         xyfile = basename(:len1)
         call xyout(npivots,fib,xyfile,'fib',fibstring,nguides,
     :      nunused,status)
      end if
      if (status .ne. af2_ok) go to 500
C
C     Write out a new input field file.  This is basically the same as
C     the old field file, except the objects which have been allocated on
C     this particular run will be zero weighted.  You can then run this
C     program on the new field file and try and allocate all the objects
C     you have thus far missed.  Search for a directory path in the new
C     file name.  If there is one, then try and create it in that place.
C     If there isn't one, then try to create it in the same directory
C     as the input field file
C
      call getval_l('Generate a new field file?','no',ifnewf,status)
      if (status .ne. af2_ok) go to 500
      if (ifnewf) then
C
C        Get the file name
C
         call getval_c('New field file name',' ',string,status)
         if (status .ne. af2_ok) go to 500
C
C        Check for file name of zero length
C
         if (chr_len(string) .eq. 0) then
            write(*,fmt='(a)') 'File name with zero length -- quitting'
            status = af2_err
            go to 500
         end if
C
C        Does it have a directory path attatched?  Again this will only
C        work for UNIX.
C
         j = 0
         len1 = chr_len(string)
         do 50 i = 1,len1
            if (string(i:i) .eq. '/') j = i
  50     continue
C
C        If a full path is specified, then keep it.  Otherwise append the
C        path to the original field file
C
         if (j .eq. 0) then
            len2 = chr_len(flddir)
            nfldfile = flddir(:len2)//string(:len1)
         else
            nfldfile = string(:len1)
         end if
C
C        Remove .fld as an extension if it's already there.
C
         len2 = chr_len(nfldfile)
         if (nfldfile(len2-3:len2) .eq. '.fld') len2 = len2 - 4
C
C        At last, generate the new field file!
C
         hush = .true.
         call newfield(fldfile,nfldfile(:len2),npivots,otype,
     :      allocated,plog,.true.,status)
         if (status .ne. af2_ok) go to 500
      end if
C
C     Close up the input field file
C
      close(fldfile)
C
C     Exit
C
  500 continue
      end
