      program af2_cscan
C+------------------------------------------------------------------------------
C
C     Name:   
C        af2_cscan
C
C     Description:
C        Main routine for allocating fibres to for autofib-2. The main
C        output is a mapping of fibre --> object for as many objects
C        in the field as the user deems possible/desirable.  An interactive
C        editor is also supplied.  This routine is similar to af2_configure
C        in many ways.  The main difference is that this routine allows you
C        to scan a number of sky position angles before doing the initial
C        placement.  As such you are not allowed to use an old configuration
C        in this routine.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        None
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        51, 52, 53
C
C     External routines called:
C        Library subaf2_configure:
C           csub_delete
C           csub_getenv
C           csub_rename
C           edit_mode
C           getval_c
C           getval_i
C           getval_l
C           getval_r
C           newfield
C           opnewfile
C           placefibs
C           rdfibinfo
C           read_objects
C           refine
C           rep_pl
C           sumweights
C           telescopeinfo
C           write_config
C           xyout
C
C        Library chr:
C           chr_len
C
C     Algorithm:
C       -- The user defines how many position angles to try.
C       -- The quality of a scan can be defined by either the total number
C          fibres placed or the total weight of the placed fibres.
C       -- There are two placement algorithms on offer (SLOW or FAST). The
C          user chooses which of these to use during the scanning process
C          and which to use when doing the final placement on the angle
C          which does the best in the scanning.
C       -- In order to help avoid collisions, the user may artifically 
C          increase the size of the fibres.
C       -- For each scan angle, the telescope information and the field file 
C          (the file with the coordinates of the objects to be allocated) is
C          read in. Then the file with the latest information on the fibre bundle
C          being used is read in. Finally a configuration is done using one of
C          the two placement algorithms and a decision is made on how good the
C          placement is for that particular angle
C       -- Once it is decided which angle is the best, a second placement is done
C          for that position angle.
C       -- The user can call the iteration algorithm in order to improve 
C          the second placement.  Once this is done, an attempt is made
C          to place any spare fibres on objects with zero weight.
C       -- The placement can be plotted and edited interactively.
C       -- An output configuration file is written. The user may optionally
C          create a file with the x,y coordinates of all the objects placed.
C          In addition, a new version of the input field file can be generated
C          which has all of the allocated objects with a weight of zero.  This
C          can then be used for a subsequent run of this program in order to
C          preferentially allocated objects which weren't allocated before.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        03 Sep 1996 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global Parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FSTA.INC'
C
C     Local parameters
C
      integer config,logfile,fldfile
      parameter (config=51,logfile=52,fldfile=53)
C
C     Local variables
C
      integer status,len1,i,j,iostat,len2,nobj,nfids,nsky
      integer otype(maxobj),sotype(maxobj),npivots,nguides,piv_r,park_r
      integer fibdims(3),fibstatus(nfibres),piv(2,nfibres)
      integer fib(2,nfibres),park(2,nfibres),allocated(nfibres)
      integer object_ref(maxobj),sky_ref(maxobj),n_obj_left,n_sky_left
      integer nall,nskip,nunused,npos,nstat,ialg,ialg1,ialg2,ipos,cfg
      integer ibest
      character dir*80,string*80,basename*80,flddir*80
      character plog(maxobj)*80,slog(maxobj)*80
      character fibstring*80,label*80,fibrefile*80,xyfile*80
      character nfldfile*80,oldname*80
      logical exist,ifplot,ifxy,ifnewf,iterate,verbose
      logical whole,replace_pa,hush
      real scales(3),field_size,object(2,maxobj),skypos(2,maxobj)
      real weight(maxobj),sweight(maxobj),fibbuf,maxnonrad,fibrelen2
      real weightsum,new_pa,dtheta,bestval
C
C     Functions
C
      integer chr_len
      real sumweights
C
C     Common blocks
C
      common /scan/ replace_pa,new_pa,hush
C
C     Initialise the status variable
C
      status = af2_ok
C
C     Get the name of the directory where the "general" files are kept
C     These include the fibre information file and the telescope information
C     file. Make sure there is a / at the end of the string.  Note this will
C     only work for UNIX boxes.
C
      call csub_getenv('WHT_AF2DATA_DIR',dir,15,80,status)
      len1 = chr_len(dir)
      if (dir(len1:len1) .ne. '/') dir(len1+1:len1+1) = '/'
C
C     Get the parameters which map the telescope distortion from telescope
C     information file.
C
      call telescopeinfo(dir,field_size,scales,status)
      if (status .ne. af2_ok) go to 500
C
C     Get the input field file.  This should be a file with the extension
C     .fld, but strip this extension off if it does (it will be added back
C     in later).  Also, keep a copy of the base name which you will use 
C     as a base name for other files.  
C
      call getval_c('Input field file',' ',string,status)
      if (status .ne. af2_ok) go to 500
      len1 = chr_len(string)
      if (string(len1-3:len1) .eq. '.fld') len1 = len1 - 4
      basename = string(1:len1)
C
C     Get the name of the directory for later use.  Note this will only work
C     for UNIX files since it searches only for '/' as the directory
C     delimiter.
C
      j = 0
      do 10 i = 1,len1
         if (string(i:i) .eq. '/') j = i
  10  continue
      if (j .gt. 0) then
         flddir = string(1:j)
      else
         flddir = ' '
      end if
C
C     Add the .fld (always lower case!) extension and try to open the 
C     field file now
C
      string(len1+1:len1+4) = '.fld'
      len1 = len1 + 4
      write(*,fmt='(a,a)') 'Input field file is: ',string(:len1)
      open(unit=fldfile,file=string(:len1),status='old',
     :   access='sequential',iostat=iostat)
      if (iostat .ne. 0) then
         status = af2_err
         write(*,fmt='(a,a)') 'Error opening file: ',string(:len1)
         go to 500
      end if
C
C     Decide whether to operate in verbose mode.  Having verbose switched
C     off should cut out most of the messages you get with the exception
C     of error messages.
C
      call getval_l('Do you want verbose output?','no',verbose,status)
      if (status .ne. af2_ok) go to 500
C
C     Decide how many scans (i.e. how many different position angles)
C
      call getval_i('How many position angles to try?','10','1','359',
     :   npos,status)
      if (status .ne. af2_ok) go to 500
      if (npos .gt. 1) then
         dtheta = 360.0/real(npos)
      else
         dtheta = 0.0
      endif
C
C     What will you use for the criterion for best configuration?  (1) The
C     total number of fibres placed or (2) the total weight of the fibres
C     placed?  
C
      write(*,fmt='(a)') ' '
      write(*,fmt='(a)') 'Which statistical measure to use?'
      write(*,fmt='(a)') '(1) The total number of fibres placed'
      write(*,fmt='(a)') '(2) The total weight of the fibres placed'
      call getval_i('Choose one','2','1','2',nstat,status)
      if (status .ne. af2_ok) go to 500
C
C     Which algorithm to use for the initial guess?
C
      write(*,fmt='(a)') ' '
      write(*,fmt='(a)') 'Which algorithm to use during the scanning'
      write(*,fmt='(a)') '(1) The FAST algorithm'
      write(*,fmt='(a)') '(2) The SLOW algorithm'
      call getval_i('Choose one','1','1','2',ialg1,status)
      if (status .ne. af2_ok) go to 500
C
C     Which algorithm to use for final placement?
C
      write(*,fmt='(a)') ' '
      write(*,fmt='(a)') 'Which algorithm to use for final placement'
      write(*,fmt='(a)') '(1) The FAST algorithm'
      write(*,fmt='(a)') '(2) The SLOW algorithm'
      call getval_i('Choose one','2','1','2',ialg2,status)
      if (status .ne. af2_ok) go to 500
C
C     Get a buffer factor for the fibre dimensions.  This can be used as 
C     insurance against collisions by augmenting the physical sizes of the 
C     button and the tubing and hence making slightly dubious fibre-object
C     combinations less likely.  Note this factor must be greater than or
C     equal to 1.  
C
      call getval_r('Buffer factor for fibre dimensions','1.00','1.00',
     :   ' ',fibbuf,status)
      if (status .ne. af2_ok) go to 500
C
C     Open a log file now...
C
      call opnewfile(basename,'log',logfile,.true.,status)
      if (status .ne. af2_ok) go to 500
C
C     Start the scanning loop.  Initialise the configuration file
C     descriptor since no configuration will be written during the
C     loop.  Also initialise the configuration quality monitor.
C
      ibest = 0
      bestval = 0.0
      replace_pa = .true.
      hush = .false.
      do 50 ipos = 1,npos+1
         if (ipos .ne. npos+1) then
            new_pa = real(ipos-1)*dtheta
            ialg = ialg1
            cfg = 0
         else
            new_pa = real(ibest-1)*dtheta
            ialg = ialg2
            call opnewfile(basename,'cfg',config,.true.,status)
            cfg = config
            if (status .ne. af2_ok) go to 500
            write(*,'(a)') ' '
            write(*,'(a,f8.2,a,f8.2)') 'Best result was:',bestval,
     :         ' for position angle',new_pa
         end if
         if (ipos .gt. 1) hush = .true.
C
C        Read the field file.  This includes reading the header information
C        (UTDATE, HA, etc), reading the object information, converting the
C        RA,DEC to x,y and sorting the objects by types.  We are finished
C        with the log file so close it up.  Finally, tell the user how
C        many objects of each type there were.  Note that for convenience
C        sake the number of fiducials is also included in the counter for
C        the number of objects, since they will be allocated simultaneously.
C
         call read_objects(fldfile,cfg,logfile,verbose,scales,
     :      field_size,nobj,nfids,nsky,nskip,object,otype,skypos,
     :      sotype,plog,slog,weight,sweight,fibstring,label,status)
         if (status .ne. af2_ok) go to 500
         if (verbose) then
            write(*,fmt='(a)') ' '
            write(*,fmt='(i3,a)') nfids,' fiducial objects'
            write(*,fmt='(i3,a)') nobj-nfids,' program objects'
            write(*,fmt='(i3,a)') nsky,' sky positions'
            write(*,fmt='(i3,a)') nskip,' objects outside of field'
            write(*,fmt='(a)') ' '
         end if
C
C        If there are fewer than 3 fiducials send out a warning message.
C
         if (nfids .lt. 3) then
            write(*,fmt='(a)') 
     :         '***Warning -- less than 3 fiducials found in field file'
            write(*,fmt='(a)') ' '
         end if
C
C        Form the name of the fibre information file from the name of the fibre
C        bundle as it was read in the header of the field file. The name of
C        the fibre information file is (always in lower case) the name of the
C        bundle with the string '_fibres.dat' appended.  Don't forget the
C        name of the directory as well
C
         len1 = chr_len(fibstring)
         len2 = chr_len(dir)
         fibrefile = dir(:len2)//fibstring(:len1)//'_fibres.dat'
         len1 = len1 + len2 + 11
C
C        See if the fibre info file exists.  If it doesn't, then signal
C        an error
C
         inquire(file=fibrefile(:len1),exist=exist)
         if (.not. exist) then
            write(*,fmt='(a,a,a)') 'The file ',fibrefile(:len1),
     :         ' does not exist'
            write(*,fmt='(a)')
     :         'Do you have the correct bundle name in field file?'
            status = af2_err
            go to 500
         end if
C
C        Read the fibre information.  This includes the number of pivots,
C        the number of guide fibres, the maximum non-radial angle the fibres
C        can subtend, the square of the fibre length, the dimensions of the
C        fibre tubing and button, the pivot and park radii, the status of
C        each fibre (whether it's disabled, etc), the pivot locations,
C        the current button locations and the park locations.
C
         call rdfibinfo(fibrefile,fibbuf,npivots,nguides,nunused,
     :      maxnonrad,fibrelen2,fibdims,piv_r,park_r,fibstatus,piv,
     :      fib,park,status)
         if (status .ne. af2_ok) go to 500
C
C        Initialise the fibre allocation record.  These will eventually 
C        say to which object a specific fibre has been allocated.  If the
C        value is zero, then the fibre is unallocated and available.  If the
C        value is negative then there is something wrong.  The fibre status 
C        (fibstatus) and the allocation record (allocated) can assume the 
C        following values initially.
C
C           fibstatus          allocated       comment
C           ------------------------------------------------------------------
C           fsta_miss            -30           Missing fibre
C           fsta_ook               0           Normal object fibre
C           fsta_fok               0           Normal guide fibre
C           fsta_odis            -10           Disabled object fibre
C           fsta_fdis            -10           Disabled guide fibre
C           fsta_odead           -20           Dead object fibre
C           fsta_fdead           -20           Dead guide fibre
C
C        Later when the fibre allocation has been done, the fibres which
C        started with values of 0 can have the reference number of an object 
C        or sky patch.  
C
         do 20 i = 1,npivots
            if ((fibstatus(i) .eq. fsta_ook) .or. 
     :         (fibstatus(i) .eq. fsta_fok)) then
               allocated(i) = 0
            else if ((fibstatus(i) .eq. fsta_odis) .or. 
     :         (fibstatus(i) .eq. fsta_fdis)) then
               allocated(i) = -10
            else if ((fibstatus(i) .eq. fsta_odead) .or. 
     :         (fibstatus(i) .eq. fsta_fdead)) then
               allocated(i) = -20
            else if (fibstatus(i) .eq. fsta_miss) then
               allocated(i) = -30
            end if
  20     continue
C
C        The array object_ref contains the number of each object.  Initially
C        this is just the number 1,2,3...nobj.  But this array will be 
C        swapped around a lot, so referencing object information won't be 
C        quite so trivial.  Do the same for the sky patches.  
C
         do 30 i = 1,nobj
            object_ref(i) = i
  30     continue
         do 40 i = 1,nsky
            sky_ref(i) = i
  40     continue
C
C        The variables n_obj_left and n_sky_left denote the number of objects
C        and skies which haven't yet been covered by a fibre.  We initialise
C        them to the number of objects and skies read in the field file.
C
         n_obj_left = nobj
         n_sky_left = nsky
C
C        If this is the last time through, then rename the field file
C        by appending the suffix 'OLD' to the base name.  Then create a 
C        new field file with the correct rotation in it.
C
C
         if (ipos .eq. npos+1) then
            len1 = chr_len(basename)
            oldname = basename(:len1)//"OLD.fld"
            inquire(file=oldname,exist=exist)
            if (exist) then
               call csub_delete(oldname,80,status)
               if (status .ne. af2_ok) go to 500
            end if
            close(fldfile,iostat=iostat)
            nfldfile = basename(:len1)//".fld"
            call csub_rename(nfldfile,80,oldname,80,status)
            if (status .ne. af2_ok) go to 500
            len2 = chr_len(oldname)
            open(fldfile,file=oldname(:len2),status='old',
     :         access='sequential',iostat=iostat)
            if (iostat .ne. 0) then
               status = af2_err
               write(*,fmt='(a,a)') 'Error opening file: ',
     :            oldname(:len1+7)
               go to 500
            end if
            call newfield(fldfile,basename(:len1),npivots,otype,
     :         allocated,plog,.false.,status)
            if (status .ne. af2_ok) go to 500
            close(fldfile,iostat=iostat)
            open(fldfile,file=basename(:len1)//".fld",status='old',
     :         access='sequential',iostat=iostat)
            if (iostat .ne. 0) then
               status = af2_err
               write(*,fmt='(a,a)') 'Error opening file: ',
     :            basename(:len1)//".fld"
               go to 500
            end if
         end if
C
C        Write out a helpful line of information
C
         if (verbose) 
     :      write(*,fmt='(a,i3,a,i3,a)') 'Trying to configure ',nobj,
     :      ' objects and ',npivots,' pivots'
C
C        Place the fibres on the objects.
C
         call placefibs(npivots,piv,fib,fibstatus,allocated,0,verbose,
     :      n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :      fibdims,fibrelen2,ialg,.true.,nall,status)
         if (status .ne. af2_ok) go to 500
C
C        Report on the number of fibres placed and the total weight of
C        those objects
C
         weightsum = sumweights(npivots,allocated,weight)
         write(*,fmt='(a)') ' '
         if (ipos .le. npos) then
            write(*,fmt='(a,i3,a,f8.2)') 'Results for scan:',ipos,
     :         ', position angle:',new_pa
         else
            write(*,fmt='(a,f8.2)') 
     :         'Final result for position angle:',new_pa
         end if
         write(*,fmt='(a,i3)') 'Total number of objects allocated: ',
     :      nall
         write(*,fmt='(a,1pg10.3)') 
     :      'Total sum of weights of allocated objects: ',weightsum
         if (nstat .eq. 1) then
            if (real(nall) .gt. bestval) then
               bestval = real(nall)
               ibest = ipos
            endif
         else 
            if (weightsum .gt. bestval) then
               bestval = weightsum
               ibest = ipos
            end if
         end if
C
C        Now we need to do that same thing for the dedicated sky patches.
C
         call placefibs(npivots,piv,fib,fibstatus,allocated,alloc_sky,
     :      verbose,n_sky_left,skypos,sweight,slog,sotype,sky_ref,
     :      maxnonrad,fibdims,fibrelen2,ialg,.true.,nall,status)
         if (status .ne. af2_ok) go to 500
C
C        Report on the number of fibres placed on skies 
C
         write(*,fmt='(a,i3)') 
     :      'Total number of sky patches allocated: ',nall
         write(*,fmt='(a)') ' '
C
C        Give the user a listing of the number of each type of object allocated
C
         if (verbose) call rep_pl(npivots,allocated,fibstatus,status)
         if (status .ne. af2_ok) go to 500
C
C        Rewind the input field file in preparation for the next scan
C
         if (ipos .lt. npos+1) then
            rewind(fldfile,iostat=iostat)
            if (iostat .ne. 0) then
               write(*,fmt='(a)') 'Error rewinding input field file'
               status = af2_err
               go to 500
            end if
         end if
  50  continue     
      close(logfile)
C
C     In this next section the user can call an iteration procedure which will
C     attempt to improve the placement as it currently stands.  This is done
C     by trying to either park or move a fibre to a different object in the
C     hope that it will either free up more objects or it will increase the
C     total sum of weights of the objects covered.  Note also that this 
C     only moves around object+fiducial allocations.  Sky patches are 
C     left alone.
C
      call getval_l('Iterate the placement?','yes',iterate,status)
      if (status .ne. af2_ok) go to 500
C
C     If you have requested an iteration then try and refine
C     the fibre placement.  Give the user a new list of fibre placings
C
      if (iterate) then
         call refine(npivots,piv,fib,n_obj_left,maxnonrad,fibdims,
     :      fibrelen2,object,fibstatus,otype,allocated,object_ref,
     :      park,weight,plog,verbose,status)
         if (verbose) call rep_pl(npivots,allocated,fibstatus,status)
         if (status .ne. af2_ok) go to 500
      end if
C
C     If you have any objects with weights <= 0.0 and there are some spare
C     fibres to be had, then you might as well try and use them.  So use
C     the FAST algorithm to try and allocate spare fibres on such objects.
C
      write(*,fmt='(a)') ' '
      write(*,fmt='(a)') 
     :   'Trying to place spare fibres on objects with weight <= 0.0'
      call placefibs(npivots,piv,fib,fibstatus,allocated,0,verbose,
     :   n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :   fibdims,fibrelen2,1,.false.,nall,status)
      if (status .ne. af2_ok) go to 500
C
C     Give the user one last listing of the number of placed objects...
C
      call rep_pl(npivots,allocated,fibstatus,status)
      if (status .ne. af2_ok) go to 500
C
C     Now give the user the opportunity to see the allocation and possibly
C     to edit the configuration manually.  If a plot is being done ask if
C     the user wants to see the whole plate (out to the pivot circle).  
C     Usually that should not be the case.
C
      call getval_l('Plot configuration to allow for editing?','yes',
     :   ifplot,status)
      if (ifplot) then
         call getval_l('Show whole plate?','no',whole,status)
         call edit_mode(nobj,npivots,nsky,object,object_ref,otype,
     :      plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,
     :      piv,park,fibstatus,allocated,label,piv_r,park_r,field_size,
     :      maxnonrad,fibrelen2,fibdims,whole,status)
      end if
      if (status .ne. af2_ok) go to 500
C
C     Ok, that's your last chance of placing fibres for this run.  Write
C     out the configuration (.cfg) file and close it up.
C
      call write_config(config,npivots,allocated,fib,plog,slog,
     :   fibstatus,nfids,nobj,otype,scales,status)
      if (status .ne. af2_ok) go to 500
      close(config)
C
C     Write out a file with the x,y coordinates as they have been calculated
C     in this routine.  This isn't necessary for observing, but is quite
C     useful for comparison with the results of af2_setup.
C
      call getval_l('Generate an (x,y) file?','no',ifxy,status)
      if (ifxy) then
         len1 = chr_len(basename)
         xyfile = basename(:len1)
         call xyout(npivots,fib,xyfile,'fib',fibstring,nguides,
     :      nunused,status)
      end if
      if (status .ne. af2_ok) go to 500
C
C     Write out a new input field file.  This is basically the same as
C     the old field file, except the objects which have been allocated on
C     this particular run will be zero weighted.  You can then run this
C     program on the new field file and try and allocate all the objects
C     you have thus far missed.  Search for a directory path in the new
C     file name.  If there is one, then try and create it in that place.
C     If there isn't one, then try to create it in the same directory
C     as the input field file
C
      call getval_l('Generate a new field file?','no',ifnewf,status)
      if (status .ne. af2_ok) go to 500
      if (ifnewf) then
C
C        Get the file name
C
         call getval_c('New field file name',' ',string,status)
         if (status .ne. af2_ok) go to 500
C
C        Check for file name of zero length
C
         if (chr_len(string) .eq. 0) then
            write(*,fmt='(a)') 'File name with zero length -- quitting'
            status = af2_err
            go to 500
         end if
C
C        Does it have a directory path attatched?  Again this will only
C        work for UNIX.
C
         j = 0
         len1 = chr_len(string)
         do 60 i = 1,len1
            if (string(i:i) .eq. '/') j = i
  60     continue
C
C        If a full path is specified, then keep it.  Otherwise append the
C        path to the original field file
C
         if (j .eq. 0) then
            len2 = chr_len(flddir)
            nfldfile = flddir(:len2)//string(:len1)
         else
            nfldfile = string(:len1)
         end if
C
C        Remove .fld as an extension if it's already there.
C
         len2 = chr_len(nfldfile)
         if (nfldfile(len2-3:len2) .eq. '.fld') len2 = len2 - 4
C
C        At last, generate the new field file!
C
         call newfield(fldfile,nfldfile(:len2),npivots,otype,
     :      allocated,plog,.true.,status)
         if (status .ne. af2_ok) go to 500
      end if
C
C     Close up the input field file
C
      close(fldfile)
C
C     Exit
C
  500 continue
      end
