      logical function boundary(x,y,field_size,verbose)
C+------------------------------------------------------------------------------
C
C     Name:   
C        boundary
C
C     Invocation:
C        value = boundary(x,y,field_size,verbose)
C
C     Description:
C        An object or sky patch with plate coordinates x,y is checked
C        to see if it falls within a circular field of radius 'field_size'.
C        If the verbose flag is set, then a message is displayed for an
C        object which falls outside of the field.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        x:  Real (>)
C           X coordinate of the object
C        y:  Real (>)
C           Y coordinate of the object
C        field_size: Real (>)
C           Radius of the field
C        verbose: Logical (>)
C           True if the user wants a message when an object falls out
C           of the field.
C
C     Returns:
C        boundary: Logical
C           True if the object is in the field.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- The square of the distance of the object from the centre is
C           calculated and compared with the radius of the field.
C        -- If the object is inside the field, then the routine exits with
C           a value of true.
C        -- If the object is outside the field, then the routine exits with
C           a value of false.  If the verbose flag is set, a message about
C           such an object is sent to the user.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C 
      real x
      real y
      real field_size
      logical verbose
C
C     Local variables
C
      logical keep
      real rad2
C
C     Get the square of the distance from the centre (center is at 0,0)
C
      rad2 = x*x + y*y
C
C     If the object is outside the field radius, then send a message
C     if you are in verbose mode
C
      if (rad2 .gt. field_size**2) then
         keep = .false.
         if (verbose)
     :      write(*,fmt='(a,f10.2,1x,f10.2)') 
     :      'Object outside field radius ',x,y
      else
         keep = .true.
      end if
C
C     Return function value and exit
C
      boundary = keep
      end
