      subroutine edit_mode(nobj,npivots,nsky,object,object_ref,
     :   otype,plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,
     :   slog,fib,piv,park,fibstatus,allocated,label,piv_r,park_r,
     :   field_size,maxnonrad,fibrelen2,fibdims,whole,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        edit_mode
C
C     Invocation:
C        call edit_mode(nobj,npivots,nsky,object,object_ref,otype,
C           plog,weight,n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,
C           piv,park,fibstatus,allocated,label,piv_r,park_r,
C           field_size,maxnonrad,fibrelen2,fibdims,whole,status)
C
C     Description:
C        The incoming configuration is edited interactively.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nobj:  Integer (>)
C           The total number of objects
C        npivots:  Integer (>)
C           The total number of pivots
C        nsky:  Integer (>)
C           The total number of sky patches
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        plog:  Character array (>)
C           The log of information on each of the objects
C        weight:  Real array (>)
C           The weight assigned to each object
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        sky_ref:  Integer array (!)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        n_sky_left:  Integer (!)
C           The number of sky patches which have not yet been allocated
C        slog:  Character array (>)
C           The log of information on each of the sky patches
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        park:  Integer array (>)
C           The park position for each of the fibres
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        label:  Character (>)
C           A character string specifying a field name label
C        piv_r:  Real (>)
C           The radius of the pivot circle
C        park_r:  Real (>)
C           The radius of the park circle
C        field_size:  Real (>)
C           The radius of the accessable field
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        whole:  Logical (>)
C           TRUE of the whole fibre plate is to be displayed.
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           edit
C           plotit
C 
C        Library pgplot:
C           pgask
C           pgbeg
C           pgend
C           pgqinf
C           pgqvp
C           pgvport
C 
C     Algorithm:
C        -- The PGPLOT device is opened and is tested to make sure it is
C           a soft device.
C        -- The viewport is defined to be the whole frame and the number
C           of device pixels on each side is requested.  The scaling
C           of the frame is defined from these values.
C        -- The initial view of the field plate is displayed with the
C           maximum x,y according to whether the user has requested to
C           see the whole plate or not
C        -- The main editing routine is called
C        -- On exit, the PGPLOT device is closed.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer nobj
      integer npivots
      integer nsky
      real object(2,*)
      integer object_ref(*)
      integer otype(*)
      character plog(*)*80
      real weight(*)
      integer n_obj_left
      real skypos(2,*)
      integer sky_ref(*)
      integer n_sky_left
      character slog(*)*80
      integer fib(2,npivots)
      integer piv(2,npivots)
      integer park(2,npivots)
      integer fibstatus(npivots)
      integer allocated(npivots)
      character label*(*)
      integer piv_r
      integer park_r
      real field_size
      real maxnonrad
      real fibrelen2
      integer fibdims(3)
      logical whole
      integer status
C
C     Local variables
C
      real pgrad,x1,x2,y1,y2,pscale(2)
      character value*3
      integer length
C
C     Functions
C
      integer pgbeg
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Set the maximum size of the plot depending upon whether the
C     user has requested to see the whole plate.
C
      if (whole) then
         pgrad = real(piv_r)
      else
         pgrad = 1.1*real(park_r)
      end if
C
C     Open the graphics device.  Note that PGPLOT 'ok' status is 1 
C     which is why we test for 1 as opposed to 0.
C
      if (pgbeg(0,'?',1,1) .ne. 1) then
         status = af2_err
         go to 500
      end if
C
C     Check that this is a soft device.  If it isn't, then signal an error
C     (The length of the string 'value' can be either 2 or 3 for NO or YES --
C     if the string length is 3 in this case, we've requested a hard device.)
C
      call pgqinf('hardcopy',value,length)
      if (length .eq. 3) then
         write(*,fmt='(a)') 
     :      'Requested PGPLOT device is not a soft device!'
         status = af2_err
         call pgend
         go to 500
      end if
C
C     Set the scaling factors for text sub-windows.  pscale(1) is the
C     x-scaling and pscale(2) is the text height.  These are normalised by
C     a mean number of pixels in each direction.  This is to attempt to 
C     ensure that letters are legible.
C
      call pgask(.false.)
      call pgvport(0.0,1.0,0.0,1.0)
      call pgqvp(3,x1,x2,y1,y2)
      pscale(1) = 780.0/(x2-x1)
      pscale(2) = 512.0/(y2-y1)
C
C     Draw up the field now
C
      call plotit(piv_r,field_size,nobj,object,npivots,park,pgrad,
     :   otype,fibstatus,nsky,skypos,.true.,.false.,pscale,piv,fib,
     :   fibdims,.true.,label,status)
      if (status .ne. af2_ok) go to 500
C
C     Now enter the main editing routine
C
      call edit(nobj,npivots,nsky,object,object_ref,otype,plog,weight,
     :   n_obj_left,skypos,sky_ref,n_sky_left,slog,fib,piv,park,
     :   fibstatus,allocated,label,piv_r,maxnonrad,fibrelen2,fibdims,
     :   field_size,pgrad,pscale,status)
      if (status .ne. af2_ok) go to 500
C
C     Close up the graphics device
C
      call pgend
C
C     Exit
C
  500 continue
      end
