      subroutine objover(nobj,object,otype,nsky,skypos,tube,button,
     :   fpos,pivpos)
C+------------------------------------------------------------------------------
C
C     Name:   
C        objover
C
C     Invocation:
C        call objover(nobj,object,otype,nsky,skypos,tube,button,
C           fpos,pivpos)
C
C     Description:
C        Determine whether a fibre covers object positions.  If so
C        then replot the objects.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nobj:  Integer (>)
C           The total number of objects
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        nsky:  Integer (>)
C           The total number of sky patches
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        tube:  Real array (>)
C           The coordinates of the four corners of the tube.  The x
C           coordinates are in the first 4 elements, the y coordinates
C           are in the second 4.
C        button:  Real array (>)
C           The coordinates of the four corners of the button.  The x
C           coordinates are in the first 4 elements, the y coordinates
C           are in the second 4.
C        fpos:  Integer array (>)
C           The current x,y position of the fibre in question
C        pivpos:  Integer array (>)
C           The x,y position of the pivot in question
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           inside
C           plotpt
C
C        Library pgplot:
C           pgbbuf
C           pgebuf
C 
C     Algorithm:
C        -- First calculate the sine and cosine of the fibre position
C           angle w.r.t. the x axis.
C        -- Rotate the coordinates of the corners of the button and
C           the tube to the position angle.
C        -- Loop for all objects and for all sky patches.  Rotate
C           the object/sky coordinates in the same way as the tube.
C           See if the rotated coordinates are within either the
C           rotated tube or button limits.
C        -- If the rotated coordinates are in fact with the tube or
C           the button limits, then replot the object.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      integer nobj
      real object(2,*)
      integer otype(*)
      integer nsky
      real skypos(2,*)
      real tube(*)
      real button(*)
      integer fpos(*)
      integer pivpos(*)
C
C     Local variables
C
      real dx,dy,r,cost,sint,button2(8),tube2(8),butlim(2,2)
      real tublim(2,2),x,y,xp,yp
      integer i
      logical cover
C
C     Functions
C  
      logical inside
C
C     Find the sine and cosine of the position angle of the fibre
C
      dx = fpos(1) - pivpos(1)
      dy = fpos(2) - pivpos(2)
      r = sqrt(dx**2 + dy**2)
      cost = dx/r
      sint = dy/r
C
C     Now rotate the tube and the buttons
C
      do 10 i = 1,4
         button2(i) = button(i)*cost + button(i+4)*sint
         button2(i+4) = -button(i)*sint + button(i+4)*cost
         tube2(i) = tube(i)*cost + tube(i+4)*sint
         tube2(i+4) = -tube(i)*sint + tube(i+4)*cost
  10  continue
C
C     Find the x',y' limits of the rotated tube and button
C
      butlim(1,1) = min(button2(1),button2(2),button2(3),button2(4))
      butlim(2,1) = max(button2(1),button2(2),button2(3),button2(4))
      butlim(1,2) = min(button2(5),button2(6),button2(7),button2(8))
      butlim(2,2) = max(button2(5),button2(6),button2(7),button2(8))
      tublim(1,1) = min(tube2(1),tube2(2),tube2(3),tube2(4))
      tublim(2,1) = max(tube2(1),tube2(2),tube2(3),tube2(4))
      tublim(1,2) = min(tube2(5),tube2(6),tube2(7),tube2(8))
      tublim(2,2) = max(tube2(5),tube2(6),tube2(7),tube2(8))
C
C     Buffer the plot output
C
      call pgbbuf
C
C     Now loop through all of the objects to see if any need replotting.
C    
      do 20 i = 1,nobj
C
C        First rotate the object position.  Default the cover logical variable
C        to be false.
C
         x = object(1,i)
         y = object(2,i)
         xp = x*cost + y*sint
         yp = -x*sint + y*cost
         cover = .false.
C
C        Now check the rotated position versus the x,y limits of the
C        tube.
C
         if (inside(xp,yp,tublim)) then
            cover = .true.
C
C        Check it against the x,y limits of the rotated button
C
         else if (inside(xp,yp,butlim)) then
           cover = .true.
         end if
C
C        If the current object is covered by either the tube or the button
C        then replot it.
C
         if (cover) call plotpt(x,y,otype(i))
  20  continue
C
C     Now do the same for the sky patches
C
      do 30 i = 1,nsky
C
C        Rotate the position
C
         x = skypos(1,i)
         y = skypos(2,i)
         xp = x*cost + y*sint
         yp = -x*sint + y*cost
         cover = .false.
C
C        Now check the rotated position versus the x,y limits of the
C        tube.
C
         if (inside(xp,yp,tublim)) then
            cover = .true.
C
C        Check it against the x,y limits of the rotated button
C
         else if (inside(xp,yp,butlim)) then
           cover = .true.
         end if
C
C        If the current object is covered by either the tube or the button
C        then replot it.
C
         if (cover) call plotpt(x,y,3)
  30  continue
C
C     End of plot buffering
C
      call pgebuf
      call pgsci(1)
C
C     Exit
C
      end

      logical function inside(x,y,limits)
C+------------------------------------------------------------------------------
C
C     Name:   
C        inside
C
C     Invocation:
C        cover = inside(x,y,limits)
C
C     Description:
C        Decide whether the given coordinates are inside a rectangle
C        defined by the given limits.  Rectangle has a position angle
C        of zero w.r.t. the x axis.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        x:  Real (>)
C           The x coordinate in question
C        y:  Real (>)
C           The y coordinate in question
C        limits:  Real array (>)
C           The x,y limits of the rectangle
C
C     Returns:
C        cover:  Logical
C           TRUE if the given x,y coordinates are within the given limits
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Look at each limit individually to see if the x,y coordinates
C           are within the box.  This should save a bit of time from 
C           the usual method of doing all four comparisons in one if
C           statement.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real x
      real y
      real limits(2,2)
C
C     Local variables
C
      logical cover
C
C     Default the return variable to false.
C
      cover = .false.
C
C     Now check the limits one at a time
C
      if (x .ge. limits(1,1)) then
         if (x .le. limits(2,1)) then
            if (y .ge. limits(1,2)) then
               if (y .le. limits(2,2)) then
                  cover = .true.
               end if
            end if
         end if
      end if
C
C     Return the value
C
      inside = cover
      end
