      subroutine place1(npivots,piv,fib,fibstatus,allocated,alloff,
     :   n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
     :   fibdims,fibrelen2,verbose,nnew,useweight,work,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        place1
C
C     Invocation:
C        call place1(npivots,piv,fib,fibstatus,allocated,alloff,
C           n_obj_left,object,weight,plog,otype,object_ref,maxnonrad,
C           fibdims,fibrelen2,verbose,nnew,useweight,work,status)
C
C     Description:
C        Place fibres on objects by minimizing the amount of space
C        which is blocked off by a fibre.  Exclude zero weighted objects
C        if you want to.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        alloff:  Integer (>)
C           An offset used in the allocated array.  This is generally
C           used to distinguish a run where the objects are sky patches
C           from one where the objects are programs+fiducials.
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        weight:  Real array (>)
C           The weight assigned to each object
C        plog:  Character array (>)
C           The log of information on each of the objects
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        verbose:  Logical (>)
C           TRUE if extra output is desired.
C        nnew:  Integer (<)
C           The number of new placements done on this run.
C        useweight:  Logical (>)
C           TRUE if you want to exclude objects which are zero weighted.
C        work:  Real array (W)
C           Workspace.  Must be have at least 2*n_obj_left elements
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           do_fibre_check
C           ssort
C 
C        Library chr:
C           chr_dcwrd
C           chr_len
C
C        Library slalib:
C           sla_range
C 
C     Algorithm:
C        -- Loop through all of the pivots or until there are no more 
C           objects left. Divide the circle of fibres into quadrants 
C           and loop through the fibres stepping around the quadrants
C           on each iteration.  Loop backwards through the pivots so that
C           you are going in an anti-clockwise sense. 
C        -- For the current fibre, find the angle which it would subtend
C           to each of the objects.  Sort this into order of increasing
C           angle
C        -- Look at the objects in order of the biggest angle first.  This
C           is so that you block off as few objects as possible, given that
C           you are moving in the anti-clockwise sense around the pivots.
C        -- If the current object-fibre combination doesn't clash with any
C           of those already placed, then accept the placement and move
C           onto the next pivot.  If it does clash, then keep testing with
C           different objects until you either get a good placement or you
C           run out of possible objects for that fibre.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FCHK.INC'
C
C     Parameters
C
      integer npivots
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer fibstatus(npivots)
      integer allocated(npivots)
      integer alloff
      integer n_obj_left
      real object(2,*)
      real weight(*)
      character plog(*)*80
      integer otype(*)
      integer object_ref(*)
      real maxnonrad
      integer fibdims(3)
      real fibrelen2
      logical verbose
      integer nnew
      logical useweight
      real work(2,*)
      integer status
C
C     Local parameters
C
      integer maxword
      parameter (maxword=1)
      real small
      parameter (small=1.0e-6)
C
C     Local variables
C
      integer i,nuse,j,oref,offender,fstatus,nwrds,startw(maxword)
      integer stopw(maxword),lstat,len1,k,k1,iquart,qstart(4),ntest
      integer niter,ncirc
      real pivang,dx,dy,theta,piv1(2),but1(2)
      logical cont,zerowt,lookatit
      character word(maxword)*40
C
C     Functions
C
      real sla_range
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Zero the number of new placements
C
      nnew = 0
C
C     If there aren't any more objects left in the list, then escape
C
      if (n_obj_left .eq. 0) go to 500
C
C     Set up the order in which the fibres will be examined.  Decide the number
C     of fibres in a 'quadrant' and set up the starting fibre numbers for each
C
      iquart = nint(real(npivots)*0.25)
      do 10 i = 1,4
         qstart(4-i+1) = npivots - (i-i)*iquart
  10  continue
      ntest = 0
      ncirc = 5
      niter = 0
C
C     Loop over the pivots in an anti-clockwise sense.  Check first that there
C     are still objects to be covered.
C
      do while (ntest .lt. npivots)
         if (nnew .eq. n_obj_left) go to 400
C
C        'ncirc' tells you what quadrant you are in. 1=upper right, 2=lower
C        right, 3=lower left, 4 = upper left.  We are going anti-clockwise
C        so start in 4th qudrant.  'niter' tells you how many times you've
C        already been around the circle.
C
         ncirc = ncirc - 1
         if (ncirc .eq. 0) then
            ncirc = 4
            niter = niter + 1
         end if
C
C        Get a fibre number 'i'.  Check that it's not from another quadrant
C        (in the event of unequal quadrant sizes we may finish one before
C        the others).
C
         i = qstart(ncirc) - niter
         lookatit = .false.
         if (ncirc .ne. 1) then
            if (i .gt. qstart(ncirc-1)) then
               ntest = ntest + 1
               lookatit = .true.
            end if
         else
            if (i .gt. 0) then
               ntest = ntest + 1
               lookatit = .true.
            end if
         end if
C
C        Finally make sure this fibre isn't already allocated.
C
         if ((allocated(i) .eq. 0) .and. (lookatit)) then
C
C           Calculate the position angle of the current pivot
C
            pivang = atan2(real(piv(2,i)),real(piv(1,i)))
C
C           Now calculate the angle of each object with respect to the 
C           current pivot.  Make sure it's in the range of +/- pi and store
C           it in the work array.  The value 'nuse' is the number of objects
C           to consider for a particular fibre and takes into account that
C           an object in the object reference list may well have already
C           been allocated to another fibre.  Any such objects should be
C           at the end of the reference list.
C
            nuse = n_obj_left - nnew
            do 20 j = 1,nuse
               oref = object_ref(j)
               dx = real(piv(1,i)) - object(1,oref)
               dy = real(piv(2,i)) - object(2,oref)
               theta = atan2(dy,dx) - pivang
               work(1,j) = real(oref)
               work(2,j) = sla_range(theta)
  20        continue
C
C           Sort the angles into increasing order and cosort the object
C           reference array.
C
            call ssort(2,nuse,work,2)
C
C           Store the pivot position for later use
C
            piv1(1) = piv(1,i)
            piv1(2) = piv(2,i)
C
C           Now loop backwards through the objects.  This is so that objects
C           with the maximum allowable angle are used first.  This strategy
C           is designed so that a fibre placement will block as small an area
C           on the surface as possible.  As we are going around in an 
C           anti-clockwise sense, you want the pivot angles to favour the
C           anti-clockwise direction, e.g. larger angles.
C
            j = nuse
            cont = .true.
            do while ((j .ge. 1) .and. (cont))
C
C              Check that the current fibre is the right sort for the current
C              object.  Also, if you want to remove zero weighted objects,
C              check that you haven't got one here.
C
               oref = nint(work(1,j))
               zerowt = ((.not. useweight) .or. 
     :            ((weight(oref) .ge. small) .and. (useweight)))
               if ((fibstatus(i) .eq. otype(oref)) .and. 
     :            (zerowt))  then
C
C                 Next check that the angle which the fibre-object combination
C                 would have is not too big
C
                  if (abs(work(2,j)) .le. maxnonrad) then
C
C                    Finally check this fibre-object combination against all
C                    other fibres.
C
                     but1(1) = object(1,oref)
                     but1(2) = object(2,oref)
                     call do_fibre_check(npivots,piv1,but1,fibdims,i,
     :                  piv,fib,fibrelen2,maxnonrad,fibstatus,offender,
     :                  fstatus)
C
C                    If the combination makes it this far, then keep it.
C                    Update the number of new combinations and store the
C                    positional information.  Send a message to the user
C                    as well.
C
                     if (fstatus .eq. fchk_ok) then
                        nnew = nnew + 1
                        fib(1,i) = nint(object(1,oref))
                        fib(2,i) = nint(object(2,oref))
                        allocated(i) = oref + alloff
                        call chr_dcwrd(plog(oref),maxword,nwrds,startw,
     :                     stopw,word,lstat)
                        len1 = chr_len(word(1))
                        if (verbose) write(*,fmt='(a,i3,a,i3,a,a)') 
     :                     'putting fibre ',i,' on object ',oref,
     :                     ' , ',word(1)(:len1)
C
C                       Remove this object now from the current object reference
C                       list 
C
                        k = 1
                        do while ((k .le. nuse) .and. 
     :                     (oref .ne. object_ref(k)))
                           k = k + 1
                        end do
                        k1 = k
                        do 30 k = k1,nuse-1
                           object_ref(k) = object_ref(k+1)
  30                    continue
C
C                       If you have assigned an object, the break out of the
C                       do while loop
C
                        cont = .false.
                     end if
                  end if
               end if
C
C              Decrease the object index number
C
               j = j - 1
            end do          
C
C        End of "if (allocated(i)..." condition
C
         end if
C
C     End of pivot loop
C
      end do
C
C     Tidy and exit.  Decrease the number of unallocated objects by the
C     number of allocations done in this run.  
C
  400 continue
      n_obj_left = n_obj_left - nnew
  500 continue
      end
