      subroutine plotit(piv_r,field_size,n_objects,object,npivots, 
     :   park,pgrad,otype,fibstatus,nsky,skypos,init,hard,pscale,
     :   piv,fib,fibdims,drawfib,label,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        plotit
C
C     Invocation:
C        call plotit(piv_r,field_size,n_objects,object,npivots, 
C           park,pgrad,otype,fibstatus,nsky,skypos,init,hard,pscale,
C           piv,fib,fibdims,drawfib,label,status)
C
C     Description:
C        The fibre field and all objects are plotted.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        piv_r:  Real (>)
C           The radius of the pivot circle
C        field_size:  Real (>)
C           The radius of the accessable field
C        n_objects:  Integer (>)
C           The total number of objects
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        npivots:  Integer (>)
C           The total number of pivots
C        park:  Integer array (>)
C           The park position for each of the fibres
C        pgrad:  Real (>)
C           The maximum radius of the field to be displayed
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        nsky:  Integer (>)
C           The total number of sky patches
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        init:  Logical (>)
C           TRUE if you want to initialise the plot
C        hard:  Logical (>)
C           TRUE if this is a hard copy
C        pscale:  Real array (>)
C           Plot scaling factors: 1 -- the linear scale, 2 -- the text height
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        drawfib:  Logical (>)
C           TRUE if the fibres are to be drawn on the plot
C        label:  Character (>)
C           A character string specifying a field name label
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           draw_fibres
C           plotpt
C           setup_plot
C 
C        Library chr:
C           chr_puti
C 
C        Library pgplot:
C           pgbbuf
C           pgebuf
C           pgqch
C           pgqwin
C           pgsch
C           pgsci
C           pgstbg
C           pgtext
C 
C     Algorithm:
C        -- First do the plot setup (see setup_plot for more info)
C        -- If requested, draw up the fibres.
C        -- Label the pivot numbers so long as the number will appear
C           within the plot limits.  Erase an area where the text will
C           appear so that it stands out a bit better.  The label will
C           appear at the park radius.
C        -- The objects are plotted in colours and with markers which 
C           depend on their type.  Program objects are plotted as red plus
C           signs, fiducials are plotted as cyan asterisks and sky patches
C           are plotted as magenta circles.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      integer piv_r
      real field_size
      integer n_objects
      integer npivots
      integer park(2,npivots)
      integer otype(*)
      integer fibstatus(npivots)
      integer nsky
      real pgrad
      real object(2,*)
      real skypos(2,*)
      logical init
      logical hard
      real pscale(2)
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer fibdims(3)
      logical drawfib
      character*(*) label
      integer status
C
C     Local variables
C 
      integer k,len1
      character string*5
      real chsize,plotlim(4),xp,yp
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Open the plot device and draw everything except the fibres 
C     and the objects.
C
      call setup_plot(piv_r,field_size,pgrad,init,hard,pscale,label,
     :   status)
C
C     Draw the fibres now
C
      if (drawfib) call draw_fibres(piv,fib,npivots,fibdims,fibstatus, 
     :   status)
      if (status .ne. af2_ok) go to 500
C
C     Label the pivot points with the pivot numbers. Set a small 
C     character height for the numbers and set a text background colour
C     index.
C
      call pgqch(chsize)
      call pgsch(0.4)
      call pgstbg(0)
      call pgsci(1)
C
C     Get the window limits in world coordinates
C
      call pgqwin(plotlim(1),plotlim(2),plotlim(3),plotlim(4))
C
C     Now loop for each pivot and draw a fibre number as long as it isn't
C     outside the plotting box.
C
      do 10 k = 1, npivots
C
C        Get a location for each of the numbers
C
         xp = real(park(1,k))
         yp = real(park(2,k))
C
C        Only continue this if the number will be in the plotting box
C
         if ((xp .ge. plotlim(1)) .and. (xp .le. plotlim(2)) .and.
     :      (yp .ge. plotlim(3)) .and. (yp .le. plotlim(4))) then
C
C           Form the string with the pivot number.  Append and 'F' if
C           this is a fiducial fibre
C
            len1 = 0
            call chr_puti(k,string,len1)
            if ((fibstatus(k) .eq. fsta_fok) .or. 
     :         (fibstatus(k) .eq. fsta_fdis) .or.
     :         (fibstatus(k) .eq. fsta_fdead)) 
     :         call chr_putc('F',string,len1)
C
C           Write the string out
C
            call pgtext(xp,yp,string(:len1))
         end if
 10   continue
C
C     Reset the character size
C
      call pgsch(chsize)
C 
C     Plot objects in a colour and with a marker appropriate for their types.
C     Program objects are red plus signs, fiducials are cyan asterisks
C
      call pgbbuf
      do 20 k = 1, n_objects
         call plotpt(object(1,k),object(2,k),otype(k))
  20  continue
C
C     Do a similar plot for sky patches.  They are plotted as magenta circles
C 
      do 30 k = 1, nsky
         call plotpt(skypos(1,k),skypos(2,k),3)
 30   continue
      call pgebuf
C
C     Reset the colour index 
C
      call pgsci(1)
C
C     Exit
C
  500 continue
      end

      subroutine plotpt(x,y,otype)
C+------------------------------------------------------------------------------
C
C     Name:   
C        plotpt
C
C     Invocation:
C        call plotpt(x,y,otype)
C
C     Description:
C        The x,y position is plotted with a marker and in a colour which
C        varies according to the type of object it is.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        x: Real (>)
C           The x position of the object
C        y: Real (>)
C           The y position of the object
C        otype: Integer (>)
C           The object type.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library pgplot:
C           pgpt
C           pgsci
C 
C     Algorithm:
C        -- Program objects are plotted as red plus signs.
C        -- Fiducial objects are plotted as cyan asterisks.
C        -- Sky patches are plotted as magenta circles.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      real x
      real y
      integer otype
C
C     Local variables
C
      integer ci,im
C
C     Find the correct colour and marker
C
      if (otype .eq. fsta_ook) then
         ci = 2
         im = 2
      else if (otype .eq. fsta_fok) then
         ci = 5
         im = 3
      else
         ci = 6
         im = 23
      end if
C
C     Now mark the point
C
      call pgsci(ci)
      call pgpt(1,x,y,im)
C
C     Exit
C
      end
