      subroutine rdfibinfo(fname,fibbuf,npivots,nguides,nunused,
     :   maxnonrad,fibrelen2,fibdims,piv_r,park_r,fibstatus,piv,fib, 
     :   park,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        rdfibinfo
C
C     Invocation:
C        call rdfibinfo(fname,fibbuf,npivots,nguides,nununsed,
C           maxnonrad,fibrelen2,fibdims,piv_r,park_r,fibstatus,piv,fib,
C           park,status)
C
C     Description:
C        Information about the fibre bundle is read from a file
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        fname:  Character (>)
C           Name of the fibre information file
C        fibbuf:  Real (>)
C           This is an amount of padding given to the fibre expressed
C           as a fraction of the fibre dimensions. (must be >= 1.0)
C        npivots:  Integer (<)
C           The total number of pivots
C        nguides:  Integer (<)
C           The total number of guide fibres
C        nunused:  Integer (<)
C           The total number of unused fibres
C        maxnonrad: Real (<)
C           The maximum non-radial angle each fibre is allowed to 
C           subtend (radians)
C        fibrelen2:  Real (<)
C           The square of length of a fibre
C        fibdims:  Integer array (<)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        piv_r:  Real (<)
C           The radius of the pivot circle
C        park_r:  Real (<)
C           The radius of the park circle
C        fibstatus:  Integer array (<)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        piv:  Integer array (<)
C           The x,y position of each of the pivots
C        fib:  Integer array (<)
C           The current x,y position of each fibre button
C        park:  Integer array (<)
C           The park position for each of the fibres
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        41
C
C     External routines called:
C        Library subaf2_configure:
C           readint
C           readreal
C 
C        Library chr:
C           chr_ctoi
C           chr_ctor
C           chr_dcwrd
C           chr_len
C 
C     Algorithm:
C        -- Open the file and skip the first three lines.
C        -- Read the number of pivots, guides, unused and the maximum 
C           non-radial angle. Convert the latter to radians
C        -- Read the fibre dimensions.  Multiply the tube width and the
C           button dimensions by the buffer size.
C        -- Read the fibre diameter, the pivot radius, the park radius and
C           the load radius
C        -- For each pivot, read the fibre number, its status, its pivot 
C           position and its current x,y.
C        -- Calculate the park position for each fibre.  If it's useable, the
C           the park position is at the park radius.  If not, then the park
C           position is at the load radius.  If it's missing, then the park
C           position is offcially designated as the pivot position.
C        -- If the fibre is useable, then ignore the x,y position from the
C           file and assume the fibre is parked.  For all others use the x,y
C           in the fibre information file.
C        -- Close the fibre information file.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_MATH.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C
      character fname*(*)
      real fibbuf
      integer npivots
      integer nguides
      integer nunused
      real maxnonrad
      real fibrelen2
      integer fibdims(3)
      integer piv_r
      integer park_r
      integer fibstatus(nfibres)
      integer piv(2,nfibres)
      integer fib(2,nfibres)
      integer park(2,nfibres)
      integer status
C
C     Local parameters
C
      integer fd1
      parameter (fd1=41)
      integer maxword
      parameter (maxword=6)
C
C     Local variables
C
      character string*80,word(maxword)*30
      integer startw(maxword),stopw(maxword),nwrds,lstat,fibrelen
      integer i,n,len1,nread,load_r,iostat
      real rfac(3),rad,fibrediam,x,y
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Open the fibre data file
C 
      len1 = chr_len(fname)
      open(unit=fd1,file=fname(:len1),status='OLD',access='SEQUENTIAL',
     :     iostat=iostat)
      if (iostat .ne. 0) then
         write(*,'(a,a)') 'Error opening file: ',fname(:len1)
         status = af2_err
         go to 500
      end if
C
C     Ingore the first three lines.  The first is a description of the
C     file, the second is blank and the third is a description of the bundle
C
      nread = 0
      do 10 i = 1,3
         read(unit=fd1,fmt='(a)',iostat=iostat) string
         nread = nread + 1
         if (iostat .ne. 0) then
            write(*,'(a,i3,a,a)') 'Error reading line ',nread,' of ',
     :         fname(:len1)
            status = af2_err
            go to 500
         end if
  10  continue
C
C     Read the number of pivots and compare with the total number of 
C     fibres allowed by the program.
C
      call readint(fd1,nread,npivots,status)
      if (status .ne. af2_ok) go to 500
      if (npivots .gt. nfibres ) then
         status = af2_err
         write(*,'(a,a)') 'Too many fibres specified in ',fname(:len1)
         go to 500
      end if
C
C     Read the number of guide fibres, the number of unused guide fibres and
C     the maximum non-radial angle allowed.  Change the angle to radians
C 
      call readint(fd1,nread,nguides,status)
      call readint(fd1,nread,nunused,status)
      call readreal(fd1,nread,maxnonrad,status)
      if (status .ne. af2_ok) go to 500
      maxnonrad = maxnonrad*real(degtorad)
C
C     Read fibre length, and square it well eventually be compared with
C     the square of the distance to objects.
C 
      call readint(fd1,nread,fibrelen,status)
      if (status .ne. af2_ok) go to 500
      fibrelen2 = real(fibrelen)
      fibrelen2 = fibrelen2*fibrelen2
C
C     Read the tube width, the button width and the button length.
C 
      call readint(fd1,nread,fibdims(3),status)
      call readint(fd1,nread,fibdims(2),status)
      call readint(fd1,nread,fibdims(1),status)
      if (status .ne. af2_ok) go to 500
C
C     Scale the tube and button sizes by the buffer factor
C
      do 20 i = 1,3
         fibdims(i) = nint(fibbuf*real(fibdims(i)))
  20  continue
C
C     Read the fibre diameter, the pivot radius, the park radius and the
C     load radius
C
      call readreal(fd1,nread,fibrediam,status)
      call readint(fd1,nread,piv_r,status)
      call readint(fd1,nread,park_r,status)
      call readint(fd1,nread,load_r,status)
      if (status .ne. af2_ok) go to 500
C
C     Set up three scaling factors for the park position.  The first
C     one is for good object and good fiducial fibres.  The second is
C     for missing fibres.  The third is for disabled fibres.
C
      rfac(1) = real(park_r)/real(piv_r)
      rfac(2) = 1.0
      rfac(3) = real(load_r)/real(piv_r)
C
C     Loop to read the pivot information.  Start by reading a line of data.
C
      do 30 i = 1, npivots
         read(unit=fd1,fmt='(a)',iostat=iostat) string
         nread = nread + 1
         if (iostat .ne. 0) then
            write(*,'(a,i3,a,a)') 'Error reading line ',nread,' of ',
     :         fname(:len1)
            status = af2_err
            go to 500
         end if
C
C        Decode the string and read the fibre number, its status and
C        its pivot position
C
         call chr_dcwrd(string,maxword,nwrds,startw,stopw,word,lstat)
         if (nwrds .ne. maxword) then
            write(*,'(a,i3,a,a)') 'Error reading line ',nread,' of ',
     :         fname(:len1)
            status = af2_err
            go to 500
         end if
         call chr_ctoi(word(1),n,status)
         call chr_ctoi(word(2),fibstatus(n),status)
         call chr_ctoi(word(3),piv(1,n),status)
         call chr_ctoi(word(4),piv(2,n),status)
         call chr_ctor(word(5),x,status)
         call chr_ctor(word(6),y,status)
         if (status .ne. af2_ok) then
            write(*,'(a,i3,a,a)') 'Error decoding line ',nread,' of ',
     :         fname(:len1)
            go to 500
         end if
C
C        Calculate the park position for a fibre based on its status.
C        Disabled fibres are parked at the load position.
C        Missing fibres are parked at the pivot (officially). All the 
C        rest are parked at the park radius.
C
         if ((fibstatus(n) .eq. fsta_ook) .or. 
     :      (fibstatus(n) .eq. fsta_fok)) then
            rad = rfac(1)
         else if (fibstatus(n) .eq. fsta_miss) then
            rad = rfac(2)
         else
            rad = rfac(3)
         end if
         park(1,n) = nint(real(piv(1,n))*rad)
         park(2,n) = nint(real(piv(2,n))*rad)
C
C        Put all disabled and missing fibres at the location specified in
C        the fibre info file. Put all the rest at the park position
C
         if ((fibstatus(n) .eq. fsta_ook) .or. 
     :      (fibstatus(n) .eq. fsta_fok)) then
            fib(1,n) = park(1,n)
            fib(2,n) = park(2,n)
         else
            fib(1,n) = nint(x)
            fib(2,n) = nint(y)
         end if
  30  continue
C
C     Close up the file
C
      close(fd1)
C 
C     Exit
C
  500 continue
      end
