      subroutine rdhead(fdfld,fdlog,fdconf,modtar,fldlabel,date,
     : equinox,hour_ang,ih,imt,st,ideg,ima,sa,centre,sky_pa,bundle,
     : nread,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        rdhead
C
C     Invocation:
C        call rdhead(fdfld,fdlog,fdconf,modtar,fldlabel,date,equinox,
C           hour_ang,ih,imt,st,ideg,ima,sa,centre,sky_pa,bundle,nread,status)
C
C     Description:
C        Read the header of a field file
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        fdfld:  Integer (>)
C           The logical unit of the field file
C        fdlog:  Integer (>)
C           The logical unit of the log file.  If this is zero, then no log
C           will be written
C        fdconf:  Integer (>)
C           The logical unit of the output config file. If this is zero, then
C           no information will be written to the config file.
C        modtar:  Logical (>)
C           TRUE if you wish to modify the TARGET name in the output
C           configuration file.  This is useful if the output configuration
C           file (specified by the descriptor fdconf) is really a new output
C           field file.
C        fldlabel:  Character (<)
C           The field name label
C        date:  Real (<)
C           The Julian date of the observation
C        equinox:  Real(<)
C           The equinox of the coordinates
C        hour_ang:  Real (<)
C           The hour angle (sideral time - RA) of the field centre
C        ih,imt,st:  Integer, Integer, Real (<)
C           The RA of the field centre expressed as numbers
C        ideg,ima,sa:  Integer, Integer, Real (<)
C           The DEC of the field centre expressed as numbers
C        centre:  Character (<)
C           The RA and DEC of the field centre expressed as a character string
C        sky_pa:  Real (<)
C           The sky position angle
C        bundle:  Character (<)
C           The name of the fibre bundle
C        nread:  Integer (!)
C           The number of lines read so far from the field file
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           headchk
C 
C        Library chr:
C           chr_ldblk
C           chr_len
C           chr_ucase
C 
C     Algorithm:
C        -- Read a line and convert it to upper case.  If the line is 
C           blank or begins with an asterisk, skip it and go on to the 
C           next one. 
C        -- Look for the fields TARGET, UTDATE, EQUINOX, HA, CENTRE,
C           SKYPA, FIBRES.  If one of them is found, read the value of the
C           keyword and write it out to the terminal.  Also write it out
C           to the config file if you want to.  Keep tabs on what line you've
C           just read and issue an error message about the current line if
C           you can't read the keyword.
C        -- As soon as you find a line which is neither blank, a comment nor
C           has one of the above keywords, then this is a data line.  Stop
C           reading at this point.
C        -- Check to make sure all of the above fields have been defined
C           once and only once.
C        -- Rewind the input file and place it to the location of the first
C           non-header line.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C        02 May 1996 (JRL):
C           Added an extra message to tell where we stopped reading the input
C           field file in the event that there was an error detected by
C           headchk.  Should help users discover whether one of their header
C           parameters is illegal.
C        02 Sep 1996 (JRL):
C           Added ability to ignore input value of SKY_PA.  Involved adding
C           common block 'scan'
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer fdfld
      integer fdlog
      integer fdconf
      logical modtar
      character fldlabel*(*)
      real date
      real equinox
      real hour_ang
      integer ih,imt,ideg,ima
      real st,sa
      character centre*(*)
      real sky_pa
      character bundle*(*)
      integer nread
      integer status
C
C     Local variables
C
      logical loghead,cont,newconf,replace_pa,hush
      integer field_def,date_def,equinox_def,ha_def,centre_def
      integer sky_pa_def,bundle_def,len1,lenf,nerr,i,istat,len2
      character line*80,fname*80,data*80,upline*80,temp*80
      real new_pa
C
C     Functions
C
      integer chr_len
C
C     Common blocks
C
      common /scan/ replace_pa,new_pa,hush
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     See if you want to log the header information
C
      loghead = .true.
      if (fdlog .le. 0) loghead = .false.
C
C     See if you are adding this information to a new configure file
C
      newconf = .true.
      if (fdconf .le. 0) newconf = .false.
C
C     What is the name of the input file?
C
      inquire(unit=fdfld,name=fname)
      lenf = chr_len(fname)
C
C     Flags for header item definition
C
      field_def = 0
      date_def = 0
      equinox_def = 0
      ha_def = 0
      centre_def = 0
      sky_pa_def = 0
      bundle_def = 0
C
C     Now read the header.  Do this loop until you can't find anything
C     that is a header (contains a header keyword), is a comment (starts
C     with an asterisk in column 1) or is a blank line (string length of
C     zero.  Count the number of lines you read.
C
      nread = 0
      cont = .true.
      if (.not. hush) write(*,fmt='(a)') ' '
      do while (cont)
C
C        Read a line and log it if you want...
C
         read(unit=fdfld,fmt='(a)',iostat=istat) line
         nread = nread + 1
         if (istat .ne. 0) then
            write(*,fmt='(a,a,a,i4)') 'Error reading file: ',
     :      fname(:lenf),' at line ',nread
            status = af2_err
            go to 500
         end if
         if (loghead) write(unit=fdlog,fmt='(a)') line
         len1 = chr_len(line)
         upline = line(:len1)
         call chr_ucase(upline)
C
C        Trap for blank lines.  Skip over them
C
         if (len1 .gt. 0) then
C
C           Now look for one of the header items.  
C
C           The field label
C
            if (upline(1:6) .eq. 'TARGET') then
               fldlabel = upline(7:80)
               call chr_ldblk(fldlabel)
               field_def = field_def + 1
               len2 = chr_len(fldlabel)
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'Field label is ',fldlabel(:len2)
               if (newconf) then
                  if (modtar) then
                     temp = line(7:80)
                     call chr_ldblk(temp)
                     write(unit=fdconf,fmt='(a,1x,a)')
     :                  'TARGET','NEW'//temp(1:chr_len(temp))
                  else
                     write(unit=fdconf,fmt='(a)') line
                  end if
               end if
C
C           The UT date.
C
            else if (upline(1:6) .eq. 'UTDATE') then
               data = upline(7:80)
               call chr_ldblk(data)
               date_def = date_def + 1
               len2 = chr_len(data)
               read(data,fmt=*,iostat=istat) date
               if (istat .ne. 0) then
                  write(*,fmt='(a,a)') 'Error reading UTDATE in ',
     :               fname(:lenf)
                  status = af2_err
                  go to 500
               end if
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'UT date of observation is ',data(:len2)
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           The equinox of the coordinates
C
            else if (upline(1:7) .eq. 'EQUINOX') then
               data = upline(8:80)
               call chr_ldblk(data)
               len2 = chr_len(data)
               equinox_def = equinox_def + 1
               read(data,fmt=*,iostat=istat) equinox
               if (istat .ne. 0) then
                  write(*,fmt='(a,a)') 'Error reading EQUINOX in ',
     :               fname(:lenf)
                  status = af2_err
                  go to 500
               end if
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'Equinox is ',data(:len2)
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           The hour angle of observation
C
            else if (upline(1:2) .eq. 'HA') then
               data = upline(3:80)
               call chr_ldblk(data)
               len2 = chr_len(data)
               ha_def = ha_def + 1
               read(data,fmt=*,iostat=istat) hour_ang
               if (istat .ne. 0) then
                  write(*,fmt='(a,a)') 'Error reading HA in ',
     :               fname(:lenf)
                  status = af2_err
                  go to 500
               end if
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'Hour angle is ',data(:len2)
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           The centre of the field
C
            else if (upline(1:6) .eq. 'CENTRE') then
               data = upline(7:80)
               call chr_ldblk(data)
               len2 = chr_len(data)
               centre = data(:len2)
               centre_def = centre_def + 1
               read(data(1:len2),fmt=*,iostat=istat) 
     :            ih,imt,st,ideg,ima,sa
               if (istat .ne. 0) then
                  write(*,fmt='(a,a)') 'Error reading CENTRE in ',
     :               fname(:lenf)
                  status = af2_err
                  go to 500
               end if
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'Field centre is ',data(:len2)
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           The sky position angle
C
            else if (upline(1:5) .eq. 'SKYPA') then
               data = upline(6:80)
               call chr_ldblk(data)
               len2 = chr_len(data)
               sky_pa_def = sky_pa_def + 1
               read(data,fmt=*,iostat=istat) sky_pa
               if (istat .ne. 0) then
                  write(*,fmt='(a,a)') 'Error reading SKYPA in ',
     :               fname(:lenf)
                  status = af2_err
                  go to 500
               end if
               if (replace_pa) then
                  sky_pa = new_pa
                  if (loghead) write(fdlog,fmt='(a,f8.2)') 
     :               '***New SKYPA ',sky_pa
               end if
               if (.not. hush) write(*,fmt='(a,f8.2)') 
     :            'Sky PA is ',sky_pa
               if (newconf) then
                  if (replace_pa) then
                     write(unit=fdconf,fmt='(a,f8.2)') 'SKYPA ',sky_pa
                  else 
                     write(unit=fdconf,fmt='(a)') line
                  end if
               end if
C
C           The fibre bundle name
C
            else if (upline(1:6) .eq. 'FIBRES') then
               bundle = upline(7:80)
               call chr_ldblk(bundle)
               bundle_def = bundle_def + 1
               len2 = chr_len(bundle)
               if (.not. hush) write(*,fmt='(a,a)') 
     :            'Fibre bundle is ',bundle(:len2)
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           Copy over any comment lines
C
            else if (upline(1:1) .eq. '*') then
               if (newconf) write(unit=fdconf,fmt='(a)') line
C
C           Anything else, and we are officially out of the header.
C           Lower the counter for the number of header lines and get
C           out of the loop
C
            else
               nread = nread - 1
               cont = .false.
            end if
         end if
      end do
      if (.not. hush) write(*,fmt='(a)') ' '
C
C     Now check to make sure everything was defined
C
      nerr = 0
      call headchk('TARGET',field_def,fname,nerr)
      call headchk('UTDATE',date_def,fname,nerr)
      call headchk('EQUINOX',equinox_def,fname,nerr)
      call headchk('HA',ha_def,fname,nerr)
      call headchk('CENTRE',centre_def,fname,nerr)
      call headchk('SKYPA',sky_pa_def,fname,nerr)
      call headchk('FIBRES',bundle_def,fname,nerr)
      if (nerr .gt. 0) then
         write(*,fmt='(a)') ' '
         write(*,fmt='(a)') 'Stopped reading at the following line:'
         write(*,fmt='(a)') line         
         status = af2_err
         go to 500
      end if
C
C     Rewind the file and place it at the end of the header
C
      rewind(fdfld)
      do 10 i = 1,nread
         read(fdfld,fmt='(a)') data
  10  continue
C
C     Exit
C
  500 continue
      end


      subroutine headchk(item,ndef,fname,nerr)
C+------------------------------------------------------------------------------
C
C     Name:   
C        headchk
C
C     Invocation:
C        call headchk(item,ndef,fname,nerr)
C
C     Description:
C        Check that a header item is defined once and only once.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        item:  Character (>)
C           The name of the header item.
C        ndef:  Integer (>)
C           The number of times that the item has been found in the file
C        fname:  Character (>) 
C           The name of the input field file
C        nerr:  Integer (!)
C           The number of field file header errors so far
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- If ndef = 1, then exit immediately
C        -- Otherwise increment the error counter and write out an
C           appropriate error message for either a non-defined item or
C           for a multiply defined item.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      character item*(*)
      integer ndef
      character fname*(*)
      integer nerr
C
C     Local variables
C
      integer len
C
C     Functions
C
      integer chr_len
C
C     If the item has been defined only once, then everything is ok
C     and we can get out of here...
C
      if (ndef .eq. 1) go to 500
C
C     Otherwise, increment the error counter
C
      nerr = nerr + 1
C
C     If the item hasn't been defined (ndef = 0) then send out an
C     appropriate error message.
C
      len = chr_len(fname)
      if (ndef .eq. 0) then
         write(*,fmt='(a,a,a)') item,' is undefined in file ',
     :      fname(:len)
C
C     If the item has been defined more than once, then send out a 
C     different message...
C
      else if (ndef .gt. 1) then
         write(*,fmt='(a,a,a)') item,' is multiply defined in file ',
     :      fname(:len)
      end if
C
C     Exit
C
  500 continue
      end
