      subroutine rdold(ocfgfile,npivots,fibstatus,verbose,fib,piv,
     :   allocated,object_ref,object,skypos,n_obj_left,n_sky_left,
     :   sky_ref,plog,slog,fibrelen2,maxnonrad,fibdims,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        rdold
C
C     Invocation:
C        call rdold(ocfgfile,npivots,fibstatus,verbose,fib,piv,
C           allocated,object_ref,object,skypos,n_obj_left,n_sky_left,
C           sky_ref,plog,slog,fibrelen2,maxnonrad,fibdims,status)
C
C     Description:
C        An old configuration file is read and used as the initial 
C        placement for the current run.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        ocfgfile:  Character (>)
C           The complete name of the old configuration file
C        npivots:  Integer (>)
C           The total number of pivots
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        verbose:  Logical (>)
C           TRUE if extra output is desired.
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        allocated:  Integer array (<)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        skypos:  Real array (>)
C           The x,y coordinates of the sky patches
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        n_sky_left:  Integer (!)
C           The number of sky patches which have not yet been allocated
C        sky_ref:  Integer array (!)
C           The list of reference numbers of the sky patches which have
C           not been allocated to a fibre
C        plog:  Character array (>)
C           The log of information on each of the objects
C        slog:  Character array (>)
C           The log of information on each of the sky patches
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        41
C
C     External routines called:
C        Library subaf2_configure:
C           do_fibre_check
C           locobj
C           rdhead
C 
C        Library chr:
C           chr_ctoi
C           chr_dcwrd
C           chr_len
C 
C     Algorithm:
C        -- Open the old configuration file and read the header.  If the
C           header items in the old configuration file differ in any way
C           from that of the input field file, then signal an error and exit.
C        -- Read the old config file until the pivot number represented in
C           the first three characters of each line of data is greater than
C           the number of pivots.
C        -- Ignore any fibres which are disabled, dead or missing
C        -- If the fibre is parked (object name 'PARKED'), then leave the 
C           allocation variable as it was.
C        -- If the fibre is one which was placed by a cursor position (object
C           name is 'PLACE_XY') then set the allocation variable to a constant.
C        -- For all other instances, the fibre has been positioned on an object.
C           The eighth word of the input line should be the object type.
C           If the object type is kosher (either 'P', 'F' or 'S') then strip
C           the object information from the data line.
C        -- Once you have finished reading the input information, then loop
C           through the pivots and make sure that none of the fibres collide.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FCHK.INC'
      include 'AF2_FSTA.INC'
C
C     Parameters
C   
      integer npivots
      integer fibstatus(npivots)
      logical verbose
      integer fib(2,npivots)
      integer piv(2,npivots)
      integer allocated(npivots)
      integer object_ref(*)
      real object(2,*)
      real skypos(2,*)
      integer n_obj_left
      integer n_sky_left
      integer sky_ref(*)
      character ocfgfile*(*)
      character plog(*)*80
      character slog(*)*80
      real fibrelen2
      real maxnonrad
      integer fibdims(3)
      integer status
C
C     Local parameters
C
      integer oconfig
      parameter (oconfig=41)
      integer maxword
      parameter (maxword=8)
      character alltypes*3
      parameter (alltypes='PFS')
C
C     Local variables
C
      integer lenf,ih,im,ideg,ima,ih2,im2,ideg2,ima2,nread,nerr,len1
      integer len2,pivnum,nwrds,startw(maxword),stopw(maxword),istat,i
      integer offender,fstatus
      character fldlabel*80,centre*80,bundle*80,fldlabel2*80,bundle2*80
      character data*80,word(maxword)*30,type*1,centre2*80
      real date,equinox,hour_ang,st,sa,sky_pa,date2,equinox2,hour_ang2
      real st2,sa2,sky_pa2,piv1(2),but1(2)
      logical cont
C     
C     Functions
C
      integer chr_len
C
C     Common block with information read from the header of the field file
C
      common /fldhead/ fldlabel,date,equinox,hour_ang,ih,im,st,
     :   ideg,ima,sa,centre,sky_pa,bundle
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Open the old configuration file and send the user a friendly message
C
      if (verbose) write(*,fmt='(a)') 'Old configuration:'
      lenf = chr_len(ocfgfile)
      open(unit=oconfig,file=ocfgfile,status='old',access='sequential',
     :   iostat=istat)
      if (istat .ne. 0) then
         status = af2_err
         write(*,fmt='(a,a)') 'Error opening file: ',ocfgfile(:lenf)
         go to 500
      end if
C
C     Read the header from the old configuration file and compare it with
C     that read from the field file.  If they differ or if there is an
C     error in reading the header, send a message and don't go any further.
C     Changing the field label is not a fatal error. Changing any of the
C     others is a fatal error
C
      call rdhead(oconfig,0,0,.false.,fldlabel2,date2,equinox2,
     :   hour_ang2,ih2,im2,st2,ideg2,ima2,sa2,centre2,sky_pa2,bundle2,
     :   nread,status)
      if (status .ne. af2_ok) go to 500
      nerr = 0
      if (date2 .ne. date) then
         write(*,'(a)') 
     :   'DATE in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      if (equinox2 .ne. equinox) then
         write(*,'(a)') 
     :   'EQUINOX in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      if (hour_ang2 .ne. hour_ang) then
         write(*,'(a)') 
     :   'HA in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      if ((ih2 .ne. ih) .or. (im2 .ne. im) .or. (st2 .ne. st) .or.
     :   (ideg2 .ne. ideg) .or. (ima2 .ne. ima) .or. (sa2 .ne. sa)) then
         write(*,'(a)') 
     :   'CENTRE in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      if (sky_pa2 .ne. sky_pa) then
         write(*,'(a)') 
     :   'SKYPA in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      len1 = chr_len(bundle)
      len2 = chr_len(bundle2)
      if (bundle(:len1) .ne. bundle2(:len2)) then
         write(*,'(a)') 
     :   'FIBRES in old config file doesn''t match that of field file'
         nerr = nerr + 1
      end if
      if (nerr .ne. 0) then
         status = af2_err
         go to 500
      end if
C
C     Now read the first line from the file and check for errors
C
      nerr = 0
      read(oconfig,fmt='(a)',iostat=istat) data
      nread = nread + 1
      if (istat .ne. 0) then
         write(*,'(a,a)') 'There is no object info in file ',
     :      ocfgfile(:lenf)
         nerr = nerr + 1
      end if
      read(data(1:3),fmt='(i3)',iostat=istat) pivnum
      if (istat .ne. 0) then
         write(*,'(a,a,a,i4)') 'Error in pivot number in ',
     :      ocfgfile(:lenf),', line ',nread
         nerr = nerr + 1
      end if
      if (nerr .ne. 0) then
         status = af2_err
         go to 500
      end if
C      
C     Now loop for the data
C
      cont = (pivnum .le. npivots)
      do while (cont)
C
C        Chuck out any fibres which are missing or disabled
C
         if ((fibstatus(pivnum) .eq. fsta_ook) .or. 
     :      (fibstatus(pivnum) .eq. fsta_fok)) then
C
C           Send a message about parked fibres (object name 'PARKED')
C
            if (data(5:10) .eq. 'PARKED') then
               if (verbose) write(*,'(a,i3,a)') 'Fibre ',pivnum,
     :            ' parked'      
C
C           If this was a cursor placed object (object name 'PLACE_XY')
C           then remember it as an object
C
            else if (data(5:12) .eq. 'PLACE_XY') then
               istat = 0
               call chr_dcwrd(data(13:),8,nwrds,startw,stopw,word,istat)
               call chr_ctoi(word(7),fib(1,pivnum),status)
               call chr_ctoi(word(8),fib(2,pivnum),status)
               if ((nwrds .ne. 8) .or. (status .ne. af2_ok)) then
                  write(*,'(a,a,a,i4)') 'Error in file ',
     :               ocfgfile(:lenf),' line ',nread
                  status = af2_ok
               else
                  if (verbose) write(*,'(a,i3,a,i7,i7)') 'Fibre ',
     :               pivnum,' placed at',fib(1,pivnum),fib(2,pivnum)
                  allocated(pivnum) = alloc_cur
               end if
C
C           Right, what _should_ be left now are program, fiducial
C           or sky entries.  Decode the string and check on the type
C           just to be sure it's safe to continue
C
            else
               call chr_dcwrd(data(4:),8,nwrds,startw,stopw,word,istat)
               type = word(8)(1:1)
               if (nwrds .ne. 8) then
                  write(*,'(a,a,a,i4)') 'Error in ',ocfgfile(:lenf),
     :               ', line ',nread
               else if (index(alltypes,type(1:1)) .eq. 0) then
                  write(*,'(a,a,a,i4)') 'Invalid object type in ',
     :               ocfgfile(:lenf),', line ',nread
C
C              Ok, we have a kosher entry.  Now if this is either a
C              program or a fiducial, then find the corresponding object 
C              in the object log stack as it was read in from the field
C              file.
C               
               else
                  if ((type(1:1) .eq. 'F') .or. 
     :               (type(1:1) .eq. 'P')) then
                     call locobj(n_obj_left,object_ref,plog,data,
     :                  allocated,pivnum,fib,object,0,verbose,status)
                  else if (type .eq. 'S') then
                     call locobj(n_sky_left,sky_ref,slog,data,
     :                  allocated,pivnum,fib,skypos,alloc_sky,verbose,
     :                  status)
                  end if
               end if
            end if
C
C        End of missing and disabled fibres if-block
C
         end if
C
C        Now read the next line from the file and check for errors
C
         read(oconfig,fmt='(a)',iostat=istat) data
         nread = nread + 1
         if (istat .ne. 0) then
            status = af2_ok
            cont = .false.
         else
            read(data(1:3),fmt='(i3)',iostat=istat) pivnum
            if (istat .ne. 0) then
               write(*,'(a,a,a,i4)') 'Error in pivot number in ',
     :            ocfgfile(:lenf),', line ',nread
               status = af2_err
               go to 500
            end if
            cont = (pivnum .le. npivots)
         end if
C
C     End of reading loop
C
      end do
C
C     Now loop through all of the pivots and make sure that the configuration
C     is legal.  Technically this shouldn't be necessary, but you never
C     know whether perhaps the user has edited the old configure file...
C     Or perhaps the telescope field model has changed...
C
      do 10 i = 1,npivots
         if (fibstatus(i) .ne. fsta_miss) then
            piv1(1) = real(piv(1,i))
            piv1(2) = real(piv(2,i))
            but1(1) = real(fib(1,i))
            but1(2) = real(fib(2,i))
            call do_fibre_check(npivots,piv1,but1,fibdims,i,piv,fib,
     :         fibrelen2,maxnonrad,fibstatus,offender,fstatus)
            if (fstatus .ne. fchk_ok) then
               write(*,fmt='(a,i3,a,i3)') 
     :            'Collision detected between fibres ',i,' and ',
     :             offender
               status = af2_err
               go to 500
            end if
         end if
  10  continue
C
C     Close down the file and exit
C
      close(oconfig)
  500 continue
      end


                   
      subroutine locobj(nleft,ref,olog,data,allocated,pivnum,fib,
     :   obj,offset,verbose,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        locobj
C
C     Invocation:
C        call locobj(nleft,ref,olog,data,allocated,pivnum,fib,
C           obj,offset,verbose,status)
C
C     Description:
C        Locate an object in the object log string and store the information
C        on that object in the configuration arrays for a given pivot.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        nleft:  Integer (!) 
C           The number of objects remaining to be allocated
C        ref:   Integer array (!)
C           The list of reference numbers of objects which have not
C           been allocated to a fibre.
C        olog:  Character array (>)
C           The log of information on each of the objects
C        data:  Character (>)
C           The data string as read from the old configuration file.
C        allocated:  Integer array (!)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        pivnum:  Integer (>)
C           The current pivot number 
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        obj:  Real array (>)
C           The x,y coordinates of the objects 
C        offset:  Integer (>)
C           The offset to be added to the object reference number for the
C           allocation array.
C        verbose:  Logical (>)
C           TRUE if extra output is desired
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Loop through the list of all remaining unallocated objects and
C           find an entry in the log which matches the data from the old
C           configuration file.
C        -- Once a match has been found then update the allocation array,
C           remove the object reference number from the object reference 
C           list, update the fibre position array, decrement the number
C           of unallocated objects and exit the routine.
C        -- If a match is never found, then issue a warning message and
C           exit the routine.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer nleft
      integer ref(*)
      character olog(*)*80
      character data*(*)
      integer allocated(*)
      integer pivnum
      integer fib(2,*)
      real obj(2,*)
      integer offset
      logical verbose
      integer status
C
C     Local variables
C
      integer i,j,oref,len1
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Get the length of the data string
C
      len1 = chr_len(data)
C
C     Loop through all unallocated objects
C
      do 10 i = 1,nleft
C
C        Get the object reference number and see if the data string on
C        the object matches the data string from the current field file
C        object.
C
         oref = ref(i)
         if (olog(oref)(:len1-4) .eq. data(5:len1)) then
            allocated(pivnum) = oref + offset
C
C           If you get a match, then remove this object from the 
C           reference stack
C
            do 20 j = i,nleft-1
               ref(j) = ref(j+1)
  20        continue
C
C           Send a useful message
C
            if (verbose) write(*,'(a,i3,a,i3)') 'Fibre ',pivnum,
     :         ' on object ',oref
C
C           Store the object information
C
            fib(1,pivnum) = nint(obj(1,oref))
            fib(2,pivnum) = nint(obj(2,oref))
            ref(nleft) = 0
            nleft = nleft - 1
C
C           Break out of the loop
C           
            go to 500
         end if
  10  continue
C
C     If you never find a match, then send an error message
C
      if (i .gt. nleft) then
         write(*,'(a,i4)') 'Failed to find program object for fibre',
     :      pivnum
      end if
C
C     Exit
C
  500 continue
      end
