      subroutine read_objects(fldfile,config,logfile,verbose,scales,
     :   field_size,n_objects,nfids,nsky,nskip,object,otype,skypos,
     :   sotype,plog,slog,weight,sweight,fibstring,label,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        read_objects
C
C     Invocation:
C        call read_objects(fldfile,config,logfile,verbose,scales,
C           field_size,n_objects,nfids,nsky,nskip,object,otype,skypos,
C           sotype,plog,slog,weight,sweight,fibstring,label,status)
C
C     Description:
C        Read an input field file
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        fldfile:  Integer (>)
C           Logical unit of the field file
C        config:  Integer (>)
C           Logical unit of the output config file
C        logfile:  Integer (>)
C           Logical unit of the log file
C        verbose:  Logical (>)
C           TRUE if extra output is desired
C        scales:  Real array (>)
C           The field distortion coefficients
C        field_size:  Real (>)
C           The radius of the accessable field
C        nobjects:  Integer (<)
C           The total number of objects
C        nfids:  Integer (<)
C           The total number of fiducial objects (subset of nobjects)
C        nsky:  Integer (<)
C           The total number of sky patches
C        nskip:  Integer (<)
C           The total number of objects outside the field radius.
C        object:  Real array (<)
C           The x,y coordinates of the objects (program+fiducial)
C        otype:  Integer array (<)
C           The object type (i.e. whether it's a program or sky)
C        skypos:  Real array (<)
C           The x,y coordinates of the sky patches
C        sotype:  Integer array (<)
C           The sky patch type (really just set to ok value for program objects)
C        plog:  Character array (<)
C           The log of information on each of the objects
C        slog:  Character array (<)
C           The log of information on each of the sky patches
C        weight:  Real array (<)
C           The weight assigned to each object
C        sweight:  Real array (<)
C           The weight assigned to each sky patch
C        fibstring:  Character (<)
C           The fibre bundle name in lower case.
C        label:  Character (<)
C           A string with the field label and field centre
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           boundary
C           platexy
C           rd2rad
C           rdhead
C 
C        Library chr:
C           chr_ctoi
C           chr_ctor
C           chr_dcwrd
C           chr_lcase
C           chr_len
C
C        Library slalib:
C           sla_epj2d
C 
C     Algorithm:
C        -- The telescope scaling information is written to the new config
C           file as a comment.
C        -- The header of the file is read and the information stripped out.
C        -- The modified Julian date and the rotator angle are calculated
C           from the UT-date and the sky position angle respectively.
C           The position of the field centre is converted to radians.
C        -- Read the field file to the end.
C        -- Convert the input RA and DEC to radians and then to x,y positions.
C           Check that the object is within the field radius.  If it is, then
C           store the original input line, the x,y coordinates, the weight
C           and the object type.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Include files
C
      include 'AF2_PAR.INC'
      include 'AF2_MATH.INC'
      include 'AF2_FSTA.INC'
      include 'AF2_OBS.INC'
C
C     Parameters
C 
      integer fldfile
      integer config
      integer logfile
      logical verbose
      real scales(3)
      real field_size
      integer n_objects
      integer nfids
      integer nsky
      integer nskip
      real object(2,maxobj)
      integer otype(maxobj)
      real skypos(2,maxobj)
      integer sotype(maxobj)
      character plog(maxobj)*80
      character slog(maxobj)*80
      real weight(maxobj)
      real sweight(maxobj)
      character fibstring*(*)
      character label*(*)
      integer status
C
C     Local parameters
C
      integer maxword
      parameter (maxword=9)
      character alltypes*3
      parameter (alltypes='PFS')
C
C     Local variables
C
      character fname*80,fldlabel*80,centre*80,bundle*80,data*80
      character word(maxword)*30,type*1,test*90
      integer lenf,ih,im,ideg,ima,nread,len1,len2,istat,nwrds,iostat
      integer startw(maxword),stopw(maxword),irah,iram,idd,idm
      real date,equinox,hour_ang,st,sa,sky_pa,ras,ds,wt,x,y
      double precision ddate,dequinox,dhour_ang,dsc1,dsc2,dsc3
      double precision datej,cra,cdec,rot_ang,ra,dec
      logical too_many_objects,too_many_skys,first_call,cont,keep
C
C     Functions
C
      integer chr_len
      logical boundary
      double precision sla_epj2d
C
C     Common block to pass header info from the input .fld file.  This is used
C     to compare with the header info from an input .cfg file (in case you
C     are using an old version of a configure file) to make sure they
C     are the same
C
      common /fldhead/ fldlabel,date,equinox,hour_ang,ih,im,st,
     :   ideg,ima,sa,centre,sky_pa,bundle      
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Send a friendly message about the input file
C
      inquire(unit=fldfile,name=fname)
      lenf = chr_len(fname)
      if (verbose) then 
         write(*,fmt='(a)') ' '
         write(*,fmt='(a,a)') 'Reading through field data file ',
     :   fname(:lenf)
         write(*,fmt='(a)') ' '
      end if
C
C     Write telescope scales to new config file as comment lines...
C
      if (config .ne. 0) then 
         write(config,fmt='(a,a)') '*  Telescope info from ',tif
         write(config,fmt='(a,1pg10.5)') '*  Field diameter (mm) ',
     :      field_size/500.0
         write(config,fmt='(a,1pg10.5,1pg10.5,1pg10.5)')
     :      '*  Telescope scales as follows: ',
     :      scales(1)*3600.0*real(radtodeg),scales(2),scales(3)
      end if
C
C     Read the header of the input file
C
      call rdhead(fldfile,logfile,config,.false.,fldlabel,date,equinox,
     :   hour_ang,ih,im,st,ideg,ima,sa,centre,sky_pa,bundle,nread,
     :   status)
      if (status .ne. af2_ok) go to 500
C
C     Create double precision versions of some of the header items
C
      ddate = dble(date)
      dequinox = dble(equinox)
      dhour_ang = dble(hour_ang)
C
C     Same for the telescope parameters
C
      dsc1 = dble(scales(1))
      dsc2 = dble(scales(2))
      dsc3 = dble(scales(3))
C
C     Append the centre to the field label
C
      len1 = chr_len(fldlabel)
      len2 = chr_len(centre)
      label = fldlabel(:len1)//' (centre '//centre(:len2)//')'
C
C     Calculate Julian date
C
      datej = sla_epj2d(ddate)
C
C     Calculate field centre in radians
C
      call rd2rad(ih,im,st,ideg,ima,sa,centre,cra,cdec)
C
C     Calculate rotator position angle from sky p.a.
C
      rot_ang = (dble(sky_pa) - 90.0d0)*degtorad
C
C     Create a lower case version of the fibre bundle name. This
C     is used later for deciding which fiber information file to 
C     read in.
C
      fibstring = bundle
      call chr_lcase(fibstring)
C
C     Now another friendly message to the user...
C
      if (verbose) then
         write(*,'(a)') ' '
         write(*,'(a)') 'Reading in object data'
      end if
C
C     Initialise counters. n_objects refers to _both_ object
C     and fiducial entries whereas nsky refers only to specified
C     sky locations. nfids is the number of fiducial objects (also
C     included in n_objects). nskip refers to objects outside the
C     field radius
C
      n_objects = 0
      nsky = 0
      nfids = 0
      nskip = 0
C
C     Initialise flags which tell when we've overfilled the
C     the data buffers.
C
      too_many_objects = .false.
      too_many_skys = .false.
C
C     Flag for the routine PLATEXY
C
      first_call = .true.
C
C     Begin to loop through the objects.  Start by reading 
C     a line of data and writing it out to the logfile.
C
      cont = .true.
      read(unit=fldfile,fmt='(a)',iostat=iostat) data
      nread = nread + 1
      if (iostat .ne. 0) then
         write(*,'(a,a)') 'No objects in file ',fname(:lenf)
         status = af2_err
         go to 500
      end if
      do while (cont)
         if (logfile .ne. 0) write(unit=logfile,fmt='(a)') data
C
C        If this is a comment or a blank line, skip over it
C
         len1 = chr_len(data)
         if ((len1 .gt. 0) .and. (data(1:1) .ne. '*')) then
C
C           Decode the data from the line of data
C
            call chr_dcwrd(data,maxword,nwrds,startw,stopw,word,
     :         istat)
C
C           Read the values from the individual words
C
            call chr_ctoi(word(2),irah,status)
            call chr_ctoi(word(3),iram,status)
            call chr_ctor(word(4),ras,status)
            call chr_ctoi(word(5),idd,status)
            call chr_ctoi(word(6),idm,status)
            call chr_ctor(word(7),ds,status)
            type = word(8)(1:1)
            call chr_ctor(word(9),wt,status)
C
C           If there was a problem decoding the data, it was probably
C           some sort of syntax error.  Skip this line and press on.
C
            if ((nwrds .lt. maxword) .or. (status .ne. af2_ok)) then
               write(*,fmt='(a,i5)') 'Syntax error on line ',nread
               status = af2_ok
C
C           If the type doesn't correspond to one of the standard
C           types, then send an error message and press on.
C
            else if (index(alltypes,type) .eq. 0) then
               write(*,fmt='(a,i4)') 
     :            'Non-standard object type on line ',nread
C
C           If everything was read properly check that we haven't
C           overfilled the data buffers for objects.  If you have
C           overfilled it for the first time, send an error message.
C           If the sky buffers are also overfilled, then get out
C           of the loop.  Otherwise leave open the opportunity to
C           to read more sky positions
C
            else if (((type(1:1) .eq. 'P') .or. (type(1:1) .eq. 'F')) 
     :         .and. (n_objects .ge. maxobj)) then
               if (.not. too_many_objects) then
                  too_many_objects = .true.
                  write(*,'(a,i4,a)') 
     :               'Too many program/fiducial objects (max ',
     :               maxobj,')'
                  if (logfile .ne. 0) write(logfile,'(a,i4,a)') 
     :               'Too many program/fiducial objects (max ',
     :               maxobj,')'
               end if
               if (too_many_skys) cont = .false.
C
C           Do the same test for sky objects
C
            else if ((type(1:1) .eq. 'S') .and. 
     :         (nsky .gt. maxobj)) then
               if (.not. too_many_skys) then
                  too_many_skys = .true.
                  write(*,'(a,i4,a)') 'Too many sky positions (max ',
     :               maxobj,')'
                  if (logfile .ne. 0) write(logfile,'(a,i4,a)') 
     :               'Too many sky positions (max ',maxobj,')'
               end if
               if (too_many_objects) cont = .false.
C
C           Ok, assuming that you've made it through all that, then
C           convert the coordinates to radians and then to x,y.  Multiply
C           x and y by 1000 to convert it microns.  Check that the 
C           object/patch is within the defined field.
C
            else
               test = word(5)//word(6)//word(7)
               call rd2rad(irah,iram,ras,idd,idm,ds,test,ra,dec)
               call platexy(ra,dec,dequinox,cra,cdec,dhour_ang,
     :            rot_ang,datej,dsc1,dsc2,dsc3,x,y,first_call,status)
               if (status .ne. af2_ok) go to 500
               x = 1000.0*x
               y = 1000.0*y
               keep = boundary(x,y,field_size,verbose)
C
C              If this object is inside the field then put the 
C              information into the relevant place
C
               if (keep) then
                  if (type(1:1) .eq. 'S') then
                     nsky = nsky + 1
                     slog(nsky) = data
                     skypos(1,nsky) = x
                     skypos(2,nsky) = y
                     sweight(nsky) = wt
                     sotype(nsky) = fsta_ook
                  else
                     n_objects = n_objects + 1
                     plog(n_objects) = data
                     object(1,n_objects) = x
                     object(2,n_objects) = y
                     weight(n_objects) = wt
                     if (type(1:1) .eq. 'P') then
                        otype(n_objects) = fsta_ook
                     else
                        otype(n_objects) = fsta_fok
                        if (type(1:1) .eq. 'F') nfids = nfids + 1
                     end if
                  end if
               else
                  if (logfile .ne. 0) 
     :               write(logfile,'(a)') 'Object outside of field'
                  nskip = nskip + 1
               end if
            end if
C
C        End of block testing for comment or blank line
C
         end if
C
C        Read the next line from the file. Break out of the loop
C        if the end of the file has been reached.
C
         read(unit=fldfile,fmt='(a)',iostat=iostat) data
         nread = nread + 1
         if (iostat .ne. 0) cont = .false.
      end do
C
C     Exit
C
  500 continue
      end 
