      subroutine refine(npivots,piv,fib,n_obj_left,maxnonrad,
     :   fibdims,fibrelen2,object,fibstatus,otype,allocated,
     :   object_ref,park,weight,plog,verbose,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        refine
C
C     Invocation:
C        call refine(npivots,piv,fib,n_obj_left,maxnonrad,
C           fibdims,fibrelen2,object,fibstatus,otype,allocated,
C           object_ref,park,weight,plog,verbose,status)
C
C     Description:
C        The incoming configuration is edited interactively.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        piv:  Integer array (>)
C           The x,y position of each of the pivots
C        fib:  Integer array (!)
C           The current x,y position of each fibre button
C        n_obj_left:  Integer (!)
C           The number of objects which have not yet been allocated.
C        maxnonrad: Real (>)
C           The maximum non-radial angle each fibre is allowed to subtend
C        fibdims:  Integer array (>)
C           The dimensions of the fibres: 1 -- button length, 2 -- button
C           width, 3 -- tube width
C        fibrelen2:  Real (>)
C           The square of length of a fibre
C        object:  Real array (>)
C           The x,y coordinates of the objects (program+fiducial)
C        fibstatus:  Integer array (>)
C           The fibre status (i.e. whether it's useable, disabled etc.)
C        otype:  Integer array (>)
C           The object type (i.e. whether it's a program or sky)
C        allocated:  Integer array (!)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        object_ref:  Integer array (!)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        park:  Integer array (>)
C           The park position for each of the fibres
C        weight:  Real array (>)
C           The weight assigned to each object
C        plog:  Character array (>)
C           The log of information on each of the objects
C        verbose:  Logical (>)
C           TRUE if extra output is desired.
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           do_fibre_check
C           fake_scene
C           placefibs
C           ssort
C           sumweights
C 
C     Algorithm:
C        -- Loop for the number of pivots but ignore sky and cursor placed
C           fibres.  If it is ok to park the current fibre (parking it doesn't
C           conflict with another fibre) then see if you would increase the
C           total sum of weights by parking the fibre and allowing other fibres
C           to be placed. If this scenario does lead to a possibility of 
C           increased weight sum, then store this change and the amount by
C           which you would change the weight.
C        -- If there are changes which could be made, then sort these by the
C           weight change implied by the alteration.
C        -- Loop through the number of alterations in order of decreasing 
C           weight change.  Reallocate the fibres by parking the fibre in
C           question and then calling the SLOW algorithm to do a placement.
C        -- Iterate until you can change no more placements
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_FCHK.INC'
C
C     Parameters
C
      integer npivots
      integer piv(2,npivots)
      integer fib(2,npivots)
      integer n_obj_left
      real maxnonrad
      integer fibdims(3)
      real fibrelen2
      real object(2,*)
      integer fibstatus(npivots)
      integer otype(*)
      integer allocated(npivots)
      integer object_ref(*)
      integer park(2,npivots)
      real weight(*)
      character plog(*)*80
      logical verbose
      integer status
C    
C     Local variables
C
      real cur_sum,piv1(2),but1(2),dweight
      integer cur_fib,offender,ok_to_park,an_obj_left,ntry,nall,i,oref
      integer aobject_ref(maxobj),aallocated(nfibres),afib(2,nfibres)
      integer niter
      real changes(2,nfibres)
      logical cont
C
C     Functions
C
      real sumweights
C 
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Get the total weights for the objects already allocated.
C
      cur_sum = sumweights(npivots,allocated,weight)
      write(*,fmt='(a,1pg10.3)') 'Current sum of weights ',cur_sum
C
C     ***Beginning of main "while" loop
C
C     Repeat this whole procedure until you haven't been able to improve
C     the scene any further.
C
      cont = .true.
      niter = 0
      do while (cont)
         niter = niter + 1
C
C        Zero a counter which will eventually have the number of swaps which
C        we can try in order to improve the situation
C
         ntry = 0
C
C        ***Beginning of pivot loop
C
         do 10 cur_fib = 1,npivots
C
C           Only consider fibres which are already allocated to objects or
C           fiducials (skip cursor placed and sky patches).
C
            if ((allocated(cur_fib) .gt. 0) .and. 
     :         (allocated(cur_fib) .lt. alloc_sky)) then
               piv1(1) = real(piv(1,cur_fib))
               piv1(2) = real(piv(2,cur_fib))
C
C              For the first test, check that you can park the fibre 
C
               but1(1) = real(park(1,cur_fib))
               but1(2) = real(park(2,cur_fib))
               call do_fibre_check(npivots,piv1,but1,fibdims,cur_fib,
     :            piv,fib,fibrelen2,maxnonrad,fibstatus,offender,
     :            ok_to_park)
C
C              If parking is approved, see if you can improve the 
C              allocation of the objects in that neighbourhood. Use
C              the SLOW algorithm since that attempts to maximise the
C              weight of the allocated fibres.
C
               if (ok_to_park .eq. fchk_ok) then
                  call fake_scene(npivots,cur_fib,n_obj_left,park,
     :               object_ref,allocated,fib,an_obj_left,aobject_ref,
     :               aallocated,afib,status)
                  call placefibs(npivots,piv,afib,fibstatus,aallocated,
     :               0,.false.,an_obj_left,object,weight,plog,otype,
     :               aobject_ref,maxnonrad,fibdims,fibrelen2,2,
     :               .true.,nall,status)
                  if (status .ne. af2_ok) go to 500
C
C                 Look at the change in the weight.  If the total weight
C                 has increased, then mark this change
C 
                  dweight = sumweights(npivots,aallocated,weight) -
     :               cur_sum
                  if (dweight .gt. 0.0) then
                     ntry = ntry + 1
                     changes(1,ntry) = real(cur_fib)
                     changes(2,ntry) = dweight
                  end if
               end if
            end if
C
C        ***End of pivot loop
C
  10     continue
C
C        Ok, if we have any changes that can be made then sort them
C        into increasing order of the value of the change in weight
C
         if (ntry .gt. 0) then
            call ssort(2,ntry,changes,2)
C
C           Now loop in the reverse order so that you go for the
C           best transitions first.
C
            do 20 i = ntry,1,-1
               cur_fib = nint(changes(1,i))
C
C              Park that particular fibre now, add the object back
C              into the object reference list, increment the number of
C              unallocated objects and deallocate the fibre
C
               fib(1,cur_fib) = park(1,cur_fib)
               fib(2,cur_fib) = park(2,cur_fib)
               oref = allocated(cur_fib)
               object_ref(n_obj_left+1) = oref
               n_obj_left = n_obj_left + 1
               allocated(cur_fib) = 0
C
C              Now see if you can place any more fibres
C
               call placefibs(npivots,piv,fib,fibstatus,allocated,
     :            0,.false.,n_obj_left,object,weight,plog,otype,
     :            object_ref,maxnonrad,fibdims,fibrelen2,2,
     :            .true.,nall,status)
               if (status .ne. af2_ok) go to 500
  20        continue
         end if
C
C        Send the user some information and update the current sum of weights
C
         cur_sum = sumweights(npivots,allocated,weight)
         if (verbose) then
            write(*,fmt='(a,i3,a,i3)') 'Iteration: ',niter,
     :         ', number of attempted changes:',ntry
            write(*,fmt='(a,1pg10.3)') '   Sum of weights now: ',
     :         cur_sum
         end if
C
C        If you got no improvements this time, then get out of the loop
C
         cont = ntry .gt. 0
C
C     ***End of main "while" loop
C
      end do
C
C     Send the user a final message on the sum of the weights.
C
      write(*,fmt='(a,1pg10.3)') 'Final sum of weights is: ',
     :   cur_sum
     
  500 continue
      end

      subroutine fake_scene(npivots,cur_fib,n_obj_left,park,object_ref,
     :   allocated,fib,an_obj_left,aobject_ref,aallocated,afib,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        fake_scene
C
C     Invocation:
C        call fake_scene(npivots,cur_fib,n_obj_left,park,object_ref,
C           allocated,fib,an_obj_left,aobject_ref,aallocated,afib,status)
C
C     Description:
C        Create a fake configuration by copying over the current configuration
C        and then parking the current fibre.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        npivots:  Integer (>)
C           The total number of pivots
C        cur_fib:  Integer (>)
C           The pivot number of the current fibre
C        n_obj_left:  Integer (>)
C           The number of objects which have not yet been allocated.
C        park:  Integer array (>)
C           The park position for each of the fibres
C        object_ref:  Integer array (>)
C           The list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        allocated:  Integer array (>)
C           The reference number of the object which has been allocated
C           to a particular fibre.
C        fib:  Integer array (>)
C           The current x,y position of each fibre button
C        an_obj_left:  Integer (<)
C           The fake number of objects which have not yet been allocated.
C        aobject_ref:  Integer array (<)
C           The fake list of reference numbers of objects which have not 
C           been allocated to a fibre.
C        aallocated:  Integer array (<)
C           The reference number of the object which has been allocated
C           to a particular fibre in the fake scene.
C        afib:  Integer array (<)
C           The fake current x,y position of each fibre button
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Copy the allocation information over to the fake scene arrays.
C        -- Park the current fibre in the fake arrays, add the current fibre's
C           object back into the fake object reference array and increment
C           the number of objects left unallocated in the fake scene.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      integer npivots
      integer cur_fib
      integer n_obj_left
      integer park(2,npivots)
      integer object_ref(*)
      integer allocated(npivots)
      integer fib(2,npivots)
      integer an_obj_left
      integer aobject_ref(*)
      integer aallocated(npivots)
      integer afib(2,npivots)
      integer status
C
C     Local variables
C
      integer i,oref
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Copy the fibre information over
C
      do 10 i = 1,npivots
         aallocated(i) = allocated(i)
         afib(1,i) = fib(1,i)
         afib(2,i) = fib(2,i)
  10  continue
C
C     Copy over the object information
C
      an_obj_left = n_obj_left
      do 20 i = 1,n_obj_left
         aobject_ref(i) = object_ref(i)
  20  continue
C
C     Now park the current fibre
C
      afib(1,cur_fib) = park(1,cur_fib)
      afib(2,cur_fib) = park(2,cur_fib)
C
C     Finally add the current object back into the object reference list
C
      aallocated(cur_fib) = 0
      oref = allocated(cur_fib)
      aobject_ref(an_obj_left+1) = oref
      an_obj_left = an_obj_left + 1
C
C     Exit
C 
  500 continue
      end
